@@ -28,6 +28,10 @@ class DoctrineDataCollector extends DataCollector
     private $registry;
     private $connections;
     private $managers;
+
+    /**
+     * @var DebugStack[]
+     */
     private $loggers = array();
 
     public function __construct(ManagerRegistry $registry)
@@ -65,6 +69,16 @@ class DoctrineDataCollector extends DataCollector
         );
     }
 
+    public function reset()
+    {
+        $this->data = array();
+
+        foreach ($this->loggers as $logger) {
+            $logger->queries = array();
+            $logger->currentQuery = 0;
+        }
+    }
+
     public function getManagers()
     {
         return $this->data['managers'];
@@ -117,6 +131,9 @@ class DoctrineDataCollector extends DataCollector
     private function sanitizeQuery($connectionName, $query)
     {
         $query['explainable'] = true;
+        if (null === $query['params']) {
+            $query['params'] = array();
+        }
         if (!is_array($query['params'])) {
             $query['params'] = array($query['params']);
         }
@@ -156,7 +173,11 @@ class DoctrineDataCollector extends DataCollector
     private function sanitizeParam($var)
     {
         if (is_object($var)) {
-            return array(sprintf('Object(%s)', get_class($var)), false);
+            $className = get_class($var);
+
+            return method_exists($var, '__toString') ?
+                array(sprintf('Object(%s): "%s"', $className, $var->__toString()), false) :
+                array(sprintf('Object(%s)', $className), false);
         }
 
         if (is_array($var)) {
