@@ -41,7 +41,7 @@ class DoctrineDummy
     public $bar;
 
     /**
-     * @ManyToMany(targetEntity="DoctrineRelation", indexBy="guid")
+     * @ManyToMany(targetEntity="DoctrineRelation", indexBy="rguid")
      */
     protected $indexedBar;
 
@@ -55,6 +55,16 @@ class DoctrineDummy
      */
     private $time;
 
+    /**
+     * @Column(type="time_immutable")
+     */
+    private $timeImmutable;
+
+    /**
+     * @Column(type="dateinterval")
+     */
+    private $dateInterval;
+
     /**
      * @Column(type="json_array")
      */
@@ -65,6 +75,16 @@ class DoctrineDummy
      */
     private $simpleArray;
 
+    /**
+     * @Column(type="float")
+     */
+    private $float;
+
+    /**
+     * @Column(type="decimal", precision=10, scale=2)
+     */
+    private $decimal;
+
     /**
      * @Column(type="boolean")
      */
@@ -80,5 +100,10 @@ class DoctrineDummy
      */
     private $customFoo;
 
+    /**
+     * @Column(type="bigint")
+     */
+    private $bigint;
+
     public $notMapped;
 }
