@@ -14,7 +14,7 @@ namespace Symfony\Bridge\Doctrine\Validator\Constraints;
 use Symfony\Component\Validator\Constraint;
 
 /**
- * Constraint for the Unique Entity validator
+ * Constraint for the Unique Entity validator.
  *
  * @Annotation
  * @Target({"CLASS", "ANNOTATION"})
@@ -23,14 +23,21 @@ use Symfony\Component\Validator\Constraint;
  */
 class UniqueEntity extends Constraint
 {
+    const NOT_UNIQUE_ERROR = '23bd9dbf-6b9b-41cd-a99e-4844bcf3077f';
+
     public $message = 'This value is already used.';
     public $service = 'doctrine.orm.validator.unique';
     public $em = null;
+    public $entityClass = null;
     public $repositoryMethod = 'findBy';
     public $fields = array();
     public $errorPath = null;
     public $ignoreNull = true;
 
+    protected static $errorNames = array(
+        self::NOT_UNIQUE_ERROR => 'NOT_UNIQUE_ERROR',
+    );
+
     public function getRequiredOptions()
     {
         return array('fields');
