@@ -11,12 +11,15 @@
 
 namespace Symfony\Bridge\Twig\Tests\Command;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Bridge\Twig\Command\LintCommand;
 use Symfony\Component\Console\Application;
 use Symfony\Component\Console\Output\OutputInterface;
 use Symfony\Component\Console\Tester\CommandTester;
+use Twig\Loader\FilesystemLoader;
+use Twig\Environment;
 
-class LintCommandTest extends \PHPUnit_Framework_TestCase
+class LintCommandTest extends TestCase
 {
     private $files;
 
@@ -28,7 +31,7 @@ class LintCommandTest extends \PHPUnit_Framework_TestCase
         $ret = $tester->execute(array('filename' => array($filename)), array('verbosity' => OutputInterface::VERBOSITY_VERBOSE, 'decorated' => false));
 
         $this->assertEquals(0, $ret, 'Returns 0 in case of success');
-        $this->assertRegExp('/^\/\/ OK in /', trim($tester->getDisplay()));
+        $this->assertContains('OK in', trim($tester->getDisplay()));
     }
 
     public function testLintIncorrectFile()
@@ -65,15 +68,30 @@ class LintCommandTest extends \PHPUnit_Framework_TestCase
         $this->assertRegExp('/ERROR  in \S+ \(line /', trim($tester->getDisplay()));
     }
 
+    /**
+     * @group legacy
+     * @expectedDeprecation Passing a command name as the first argument of "Symfony\Bridge\Twig\Command\LintCommand::__construct" is deprecated since Symfony 3.4 and will be removed in 4.0. If the command was registered by convention, make it a service instead.
+     * @expectedException \RuntimeException
+     * @expectedExceptionMessage The Twig environment needs to be set.
+     */
+    public function testLegacyLintCommand()
+    {
+        $command = new LintCommand();
+
+        $application = new Application();
+        $application->add($command);
+        $command = $application->find('lint:twig');
+
+        $tester = new CommandTester($command);
+        $tester->execute(array());
+    }
+
     /**
      * @return CommandTester
      */
     private function createCommandTester()
     {
-        $twig = new \Twig_Environment(new \Twig_Loader_Filesystem());
-
-        $command = new LintCommand();
-        $command->setTwigEnvironment($twig);
+        $command = new LintCommand(new Environment(new FilesystemLoader()));
 
         $application = new Application();
         $application->add($command);
