@@ -15,7 +15,12 @@ use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Input\InputOption;
 use Symfony\Component\Console\Output\OutputInterface;
 use Symfony\Component\Console\Style\SymfonyStyle;
+use Symfony\Component\EventDispatcher\EventDispatcher;
+use Symfony\Component\Filesystem\Exception\IOException;
+use Symfony\Component\Filesystem\Filesystem;
+use Symfony\Component\HttpKernel\CacheClearer\CacheClearerInterface;
 use Symfony\Component\HttpKernel\KernelInterface;
+use Symfony\Component\HttpKernel\RebootableInterface;
 use Symfony\Component\Finder\Finder;
 
 /**
@@ -23,16 +28,43 @@ use Symfony\Component\Finder\Finder;
  *
  * @author Francis Besset <francis.besset@gmail.com>
  * @author Fabien Potencier <fabien@symfony.com>
+ *
+ * @final since version 3.4
  */
 class CacheClearCommand extends ContainerAwareCommand
 {
+    protected static $defaultName = 'cache:clear';
+
+    private $cacheClearer;
+    private $filesystem;
+    private $warning;
+
+    /**
+     * @param CacheClearerInterface $cacheClearer
+     * @param Filesystem|null       $filesystem
+     */
+    public function __construct($cacheClearer = null, Filesystem $filesystem = null)
+    {
+        if (!$cacheClearer instanceof CacheClearerInterface) {
+            @trigger_error(sprintf('%s() expects an instance of "%s" as first argument since Symfony 3.4. Not passing it is deprecated and will throw a TypeError in 4.0.', __METHOD__, CacheClearerInterface::class), E_USER_DEPRECATED);
+
+            parent::__construct($cacheClearer);
+
+            return;
+        }
+
+        parent::__construct();
+
+        $this->cacheClearer = $cacheClearer;
+        $this->filesystem = $filesystem ?: new Filesystem();
+    }
+
     /**
      * {@inheritdoc}
      */
     protected function configure()
     {
         $this
-            ->setName('cache:clear')
             ->setDefinition(array(
                 new InputOption('no-warmup', '', InputOption::VALUE_NONE, 'Do not warm up the cache'),
                 new InputOption('no-optional-warmers', '', InputOption::VALUE_NONE, 'Skip optional cache warmers (faster)'),
@@ -54,61 +86,82 @@ EOF
      */
     protected function execute(InputInterface $input, OutputInterface $output)
     {
-        $outputIsVerbose = $output->isVerbose();
+        // BC to be removed in 4.0
+        if (null === $this->cacheClearer) {
+            $this->cacheClearer = $this->getContainer()->get('cache_clearer');
+            $this->filesystem = $this->getContainer()->get('filesystem');
+            $realCacheDir = $this->getContainer()->getParameter('kernel.cache_dir');
+        }
+
+        $fs = $this->filesystem;
         $io = new SymfonyStyle($input, $output);
 
-        $realCacheDir = $this->getContainer()->getParameter('kernel.cache_dir');
+        $kernel = $this->getApplication()->getKernel();
+        $realCacheDir = isset($realCacheDir) ? $realCacheDir : $kernel->getContainer()->getParameter('kernel.cache_dir');
         // the old cache dir name must not be longer than the real one to avoid exceeding
         // the maximum length of a directory or file path within it (esp. Windows MAX_PATH)
         $oldCacheDir = substr($realCacheDir, 0, -1).('~' === substr($realCacheDir, -1) ? '+' : '~');
-        $filesystem = $this->getContainer()->get('filesystem');
+        $fs->remove($oldCacheDir);
 
         if (!is_writable($realCacheDir)) {
             throw new \RuntimeException(sprintf('Unable to write in the "%s" directory', $realCacheDir));
         }
 
-        if ($filesystem->exists($oldCacheDir)) {
-            $filesystem->remove($oldCacheDir);
-        }
-
-        $kernel = $this->getContainer()->get('kernel');
         $io->comment(sprintf('Clearing the cache for the <info>%s</info> environment with debug <info>%s</info>', $kernel->getEnvironment(), var_export($kernel->isDebug(), true)));
-        $this->getContainer()->get('cache_clearer')->clear($realCacheDir);
+        $this->cacheClearer->clear($realCacheDir);
 
-        if ($input->getOption('no-warmup')) {
-            $filesystem->rename($realCacheDir, $oldCacheDir);
-        } else {
-            // the warmup cache dir name must have the same length than the real one
-            // to avoid the many problems in serialized resources files
-            $realCacheDir = realpath($realCacheDir);
-            $warmupDir = substr($realCacheDir, 0, -1).('_' === substr($realCacheDir, -1) ? '-' : '_');
+        // The current event dispatcher is stale, let's not use it anymore
+        $this->getApplication()->setDispatcher(new EventDispatcher());
 
-            if ($filesystem->exists($warmupDir)) {
-                if ($outputIsVerbose) {
-                    $io->comment('Clearing outdated warmup directory...');
-                }
-                $filesystem->remove($warmupDir);
-            }
+        $containerDir = new \ReflectionObject($kernel->getContainer());
+        $containerDir = basename(dirname($containerDir->getFileName()));
 
-            if ($outputIsVerbose) {
+        // the warmup cache dir name must have the same length as the real one
+        // to avoid the many problems in serialized resources files
+        $warmupDir = substr($realCacheDir, 0, -1).('_' === substr($realCacheDir, -1) ? '-' : '_');
+
+        if ($output->isVerbose() && $fs->exists($warmupDir)) {
+            $io->comment('Clearing outdated warmup directory...');
+        }
+        $fs->remove($warmupDir);
+        $fs->mkdir($warmupDir);
+
+        if (!$input->getOption('no-warmup')) {
+            if ($output->isVerbose()) {
                 $io->comment('Warming up cache...');
             }
             $this->warmup($warmupDir, $realCacheDir, !$input->getOption('no-optional-warmers'));
 
-            $filesystem->rename($realCacheDir, $oldCacheDir);
-            if ('\\' === DIRECTORY_SEPARATOR) {
-                sleep(1);  // workaround for Windows PHP rename bug
+            if ($this->warning) {
+                @trigger_error($this->warning, E_USER_DEPRECATED);
+                $io->warning($this->warning);
+                $this->warning = null;
             }
-            $filesystem->rename($warmupDir, $realCacheDir);
         }
 
-        if ($outputIsVerbose) {
+        $containerDir = $fs->exists($warmupDir.'/'.$containerDir) ? false : $containerDir;
+
+        $fs->rename($realCacheDir, $oldCacheDir);
+        $fs->rename($warmupDir, $realCacheDir);
+
+        if ($containerDir) {
+            $fs->rename($oldCacheDir.'/'.$containerDir, $realCacheDir.'/'.$containerDir);
+            touch($realCacheDir.'/'.$containerDir.'.legacy');
+        }
+
+        if ($output->isVerbose()) {
             $io->comment('Removing old cache directory...');
         }
 
-        $filesystem->remove($oldCacheDir);
+        try {
+            $fs->remove($oldCacheDir);
+        } catch (IOException $e) {
+            if ($output->isVerbose()) {
+                $io->warning($e->getMessage());
+            }
+        }
 
-        if ($outputIsVerbose) {
+        if ($output->isVerbose()) {
             $io->comment('Finished');
         }
 
@@ -123,18 +176,24 @@ EOF
     protected function warmup($warmupDir, $realCacheDir, $enableOptionalWarmers = true)
     {
         // create a temporary kernel
-        $realKernel = $this->getContainer()->get('kernel');
-        $realKernelClass = get_class($realKernel);
-        $namespace = '';
-        if (false !== $pos = strrpos($realKernelClass, '\\')) {
-            $namespace = substr($realKernelClass, 0, $pos);
-            $realKernelClass = substr($realKernelClass, $pos + 1);
-        }
-        $tempKernel = $this->getTempKernel($realKernel, $namespace, $realKernelClass, $warmupDir);
-        $tempKernel->boot();
+        $realKernel = $this->getApplication()->getKernel();
+        if ($realKernel instanceof RebootableInterface) {
+            $realKernel->reboot($warmupDir);
+            $tempKernel = $realKernel;
+        } else {
+            $this->warning = 'Calling "cache:clear" with a kernel that does not implement "Symfony\Component\HttpKernel\RebootableInterface" is deprecated since Symfony 3.4 and will be unsupported in 4.0.';
+            $realKernelClass = get_class($realKernel);
+            $namespace = '';
+            if (false !== $pos = strrpos($realKernelClass, '\\')) {
+                $namespace = substr($realKernelClass, 0, $pos);
+                $realKernelClass = substr($realKernelClass, $pos + 1);
+            }
+            $tempKernel = $this->getTempKernel($realKernel, $namespace, $realKernelClass, $warmupDir);
+            $tempKernel->boot();
 
-        $tempKernelReflection = new \ReflectionObject($tempKernel);
-        $tempKernelFile = $tempKernelReflection->getFileName();
+            $tempKernelReflection = new \ReflectionObject($tempKernel);
+            $tempKernelFile = $tempKernelReflection->getFileName();
+        }
 
         // warmup temporary dir
         $warmer = $tempKernel->getContainer()->get('cache_warmer');
@@ -143,11 +202,25 @@ EOF
         }
         $warmer->warmUp($warmupDir);
 
+        // fix references to cached files with the real cache directory name
+        $search = array($warmupDir, str_replace('\\', '\\\\', $warmupDir));
+        $replace = str_replace('\\', '/', $realCacheDir);
+        foreach (Finder::create()->files()->in($warmupDir) as $file) {
+            $content = str_replace($search, $replace, file_get_contents($file), $count);
+            if ($count) {
+                file_put_contents($file, $content);
+            }
+        }
+
+        if ($realKernel instanceof RebootableInterface) {
+            return;
+        }
+
         // fix references to the Kernel in .meta files
         $safeTempKernel = str_replace('\\', '\\\\', get_class($tempKernel));
         $realKernelFQN = get_class($realKernel);
 
-        foreach (Finder::create()->files()->name('*.meta')->in($warmupDir) as $file) {
+        foreach (Finder::create()->files()->depth('<3')->name('*.meta')->in($warmupDir) as $file) {
             file_put_contents($file, preg_replace(
                 '/(C\:\d+\:)"'.$safeTempKernel.'"/',
                 sprintf('$1"%s"', $realKernelFQN),
@@ -155,30 +228,19 @@ EOF
             ));
         }
 
-        // fix references to cached files with the real cache directory name
-        $search = array($warmupDir, str_replace('\\', '\\\\', $warmupDir));
-        $replace = str_replace('\\', '/', $realCacheDir);
-        foreach (Finder::create()->files()->in($warmupDir) as $file) {
-            $content = str_replace($search, $replace, file_get_contents($file));
+        // fix references to container's class
+        $tempContainerClass = $tempKernel->getContainerClass();
+        $realContainerClass = $tempKernel->getRealContainerClass();
+        foreach (Finder::create()->files()->depth('<2')->name($tempContainerClass.'*')->in($warmupDir) as $file) {
+            $content = str_replace($tempContainerClass, $realContainerClass, file_get_contents($file));
             file_put_contents($file, $content);
+            rename($file, str_replace(DIRECTORY_SEPARATOR.$tempContainerClass, DIRECTORY_SEPARATOR.$realContainerClass, $file));
         }
-
-        // fix references to kernel/container related classes
-        $fileSearch = $tempKernel->getName().ucfirst($tempKernel->getEnvironment()).'*';
-        $search = array(
-            $tempKernel->getName().ucfirst($tempKernel->getEnvironment()),
-            sprintf('\'kernel.name\' => \'%s\'', $tempKernel->getName()),
-            sprintf('key="kernel.name">%s<', $tempKernel->getName()),
-        );
-        $replace = array(
-            $realKernel->getName().ucfirst($realKernel->getEnvironment()),
-            sprintf('\'kernel.name\' => \'%s\'', $realKernel->getName()),
-            sprintf('key="kernel.name">%s<', $realKernel->getName()),
-        );
-        foreach (Finder::create()->files()->name($fileSearch)->in($warmupDir) as $file) {
-            $content = str_replace($search, $replace, file_get_contents($file));
-            file_put_contents(str_replace($search, $replace, $file), $content);
-            unlink($file);
+        if (is_dir($tempContainerDir = $warmupDir.'/'.get_class($tempKernel->getContainer()))) {
+            foreach (Finder::create()->files()->in($tempContainerDir) as $file) {
+                $content = str_replace($tempContainerClass, $realContainerClass, file_get_contents($file));
+                file_put_contents($file, $content);
+            }
         }
 
         // remove temp kernel file after cache warmed up
@@ -195,14 +257,29 @@ EOF
      */
     protected function getTempKernel(KernelInterface $parent, $namespace, $parentClass, $warmupDir)
     {
+        $projectDir = '';
         $cacheDir = var_export($warmupDir, true);
         $rootDir = var_export(realpath($parent->getRootDir()), true);
         $logDir = var_export(realpath($parent->getLogDir()), true);
         // the temp kernel class name must have the same length than the real one
         // to avoid the many problems in serialized resources files
         $class = substr($parentClass, 0, -1).'_';
-        // the temp kernel name must be changed too
-        $name = var_export(substr($parent->getName(), 0, -1).'_', true);
+        // the temp container class must be changed too
+        $container = $parent->getContainer();
+        $realContainerClass = var_export($container->hasParameter('kernel.container_class') ? $container->getParameter('kernel.container_class') : get_class($parent->getContainer()), true);
+        $containerClass = substr_replace($realContainerClass, '_', -2, 1);
+
+        if (method_exists($parent, 'getProjectDir')) {
+            $projectDir = var_export(realpath($parent->getProjectDir()), true);
+            $projectDir = <<<EOF
+        public function getProjectDir()
+        {
+            return $projectDir;
+        }
+        
+EOF;
+        }
+
         $code = <<<EOF
 <?php
 
@@ -215,21 +292,27 @@ namespace $namespace
             return $cacheDir;
         }
 
-        public function getName()
-        {
-            return $name;
-        }
-
         public function getRootDir()
         {
             return $rootDir;
         }
 
+        $projectDir
         public function getLogDir()
         {
             return $logDir;
         }
 
+        public function getRealContainerClass()
+        {
+            return $realContainerClass;
+        }
+
+        public function getContainerClass()
+        {
+            return $containerClass;
+        }
+
         protected function buildContainer()
         {
             \$container = parent::buildContainer();
@@ -250,7 +333,7 @@ namespace $namespace
     }
 }
 EOF;
-        $this->getContainer()->get('filesystem')->mkdir($warmupDir);
+        $this->filesystem->mkdir($warmupDir);
         file_put_contents($file = $warmupDir.'/kernel.tmp', $code);
         require_once $file;
         $class = "$namespace\\$class";
