@@ -12,11 +12,13 @@
 namespace Symfony\Bundle\FrameworkBundle\Command;
 
 use Symfony\Bundle\FrameworkBundle\Console\Helper\DescriptorHelper;
+use Symfony\Bundle\FrameworkBundle\Controller\ControllerNameParser;
 use Symfony\Component\Console\Input\InputArgument;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Input\InputOption;
 use Symfony\Component\Console\Output\OutputInterface;
 use Symfony\Component\Console\Style\SymfonyStyle;
+use Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException;
 use Symfony\Component\Routing\RouterInterface;
 use Symfony\Component\Routing\Route;
 
@@ -25,14 +27,42 @@ use Symfony\Component\Routing\Route;
  *
  * @author Fabien Potencier <fabien@symfony.com>
  * @author Tobias Schultze <http://tobion.de>
+ *
+ * @final since version 3.4
  */
 class RouterDebugCommand extends ContainerAwareCommand
 {
+    protected static $defaultName = 'debug:router';
+    private $router;
+
+    /**
+     * @param RouterInterface $router
+     */
+    public function __construct($router = null)
+    {
+        if (!$router instanceof RouterInterface) {
+            @trigger_error(sprintf('%s() expects an instance of "%s" as first argument since Symfony 3.4. Not passing it is deprecated and will throw a TypeError in 4.0.', __METHOD__, RouterInterface::class), E_USER_DEPRECATED);
+
+            parent::__construct($router);
+
+            return;
+        }
+
+        parent::__construct();
+
+        $this->router = $router;
+    }
+
     /**
      * {@inheritdoc}
+     *
+     * BC to be removed in 4.0
      */
     public function isEnabled()
     {
+        if (null !== $this->router) {
+            return parent::isEnabled();
+        }
         if (!$this->getContainer()->has('router')) {
             return false;
         }
@@ -50,10 +80,6 @@ class RouterDebugCommand extends ContainerAwareCommand
     protected function configure()
     {
         $this
-            ->setName('debug:router')
-            ->setAliases(array(
-                'router:debug',
-            ))
             ->setDefinition(array(
                 new InputArgument('name', InputArgument::OPTIONAL, 'A route name'),
                 new InputOption('show-controllers', null, InputOption::VALUE_NONE, 'Show assigned controllers in overview'),
@@ -78,32 +104,31 @@ EOF
      */
     protected function execute(InputInterface $input, OutputInterface $output)
     {
-        $io = new SymfonyStyle($input, $output);
-
-        if (false !== strpos($input->getFirstArgument(), ':d')) {
-            $io->caution('The use of "router:debug" command is deprecated since version 2.7 and will be removed in 3.0. Use the "debug:router" instead.');
+        // BC to be removed in 4.0
+        if (null === $this->router) {
+            $this->router = $this->getContainer()->get('router');
         }
 
+        $io = new SymfonyStyle($input, $output);
         $name = $input->getArgument('name');
         $helper = new DescriptorHelper();
+        $routes = $this->router->getRouteCollection();
 
         if ($name) {
-            $route = $this->getContainer()->get('router')->getRouteCollection()->get($name);
-            if (!$route) {
+            if (!$route = $routes->get($name)) {
                 throw new \InvalidArgumentException(sprintf('The route "%s" does not exist.', $name));
             }
 
-            $this->convertController($route);
+            $callable = $this->extractCallable($route);
 
             $helper->describe($io, $route, array(
                 'format' => $input->getOption('format'),
                 'raw_text' => $input->getOption('raw'),
                 'name' => $name,
                 'output' => $io,
+                'callable' => $callable,
             ));
         } else {
-            $routes = $this->getContainer()->get('router')->getRouteCollection();
-
             foreach ($routes as $route) {
                 $this->convertController($route);
             }
@@ -119,12 +144,38 @@ EOF
 
     private function convertController(Route $route)
     {
-        $nameParser = $this->getContainer()->get('controller_name_converter');
         if ($route->hasDefault('_controller')) {
+            $nameParser = new ControllerNameParser($this->getApplication()->getKernel());
             try {
                 $route->setDefault('_controller', $nameParser->build($route->getDefault('_controller')));
             } catch (\InvalidArgumentException $e) {
             }
         }
     }
+
+    private function extractCallable(Route $route)
+    {
+        if (!$route->hasDefault('_controller')) {
+            return;
+        }
+
+        $controller = $route->getDefault('_controller');
+
+        if (1 === substr_count($controller, ':')) {
+            list($service, $method) = explode(':', $controller);
+            try {
+                return sprintf('%s::%s', get_class($this->getApplication()->getKernel()->getContainer()->get($service)), $method);
+            } catch (ServiceNotFoundException $e) {
+            }
+        }
+
+        $nameParser = new ControllerNameParser($this->getApplication()->getKernel());
+        try {
+            $shortNotation = $nameParser->build($controller);
+            $route->setDefault('_controller', $shortNotation);
+
+            return $controller;
+        } catch (\InvalidArgumentException $e) {
+        }
+    }
 }
