@@ -24,15 +24,15 @@ class TemplateLocator implements FileLocatorInterface
     protected $locator;
     protected $cache;
 
+    private $cacheHits = array();
+
     /**
-     * Constructor.
-     *
      * @param FileLocatorInterface $locator  A FileLocatorInterface instance
      * @param string               $cacheDir The cache path
      */
     public function __construct(FileLocatorInterface $locator, $cacheDir = null)
     {
-        if (null !== $cacheDir && is_file($cache = $cacheDir.'/templates.php')) {
+        if (null !== $cacheDir && file_exists($cache = $cacheDir.'/templates.php')) {
             $this->cache = require $cache;
         }
 
@@ -42,8 +42,6 @@ class TemplateLocator implements FileLocatorInterface
     /**
      * Returns a full path for a given file.
      *
-     * @param TemplateReferenceInterface $template A template
-     *
      * @return string The full path for the file
      */
     protected function getCacheKey($template)
@@ -71,12 +69,15 @@ class TemplateLocator implements FileLocatorInterface
 
         $key = $this->getCacheKey($template);
 
+        if (isset($this->cacheHits[$key])) {
+            return $this->cacheHits[$key];
+        }
         if (isset($this->cache[$key])) {
-            return $this->cache[$key];
+            return $this->cacheHits[$key] = realpath($this->cache[$key]) ?: $this->cache[$key];
         }
 
         try {
-            return $this->cache[$key] = $this->locator->locate($template->getPath(), $currentPath);
+            return $this->cacheHits[$key] = $this->locator->locate($template->getPath(), $currentPath);
         } catch (\InvalidArgumentException $e) {
             throw new \InvalidArgumentException(sprintf('Unable to find template "%s" : "%s".', $template, $e->getMessage()), 0, $e);
         }
