@@ -18,7 +18,6 @@ class FirewallEntryPointTest extends WebTestCase
     public function testItUsesTheConfiguredEntryPointWhenUsingUnknownCredentials()
     {
         $client = $this->createClient(array('test_case' => 'FirewallEntryPoint'));
-        $client->insulate();
 
         $client->request('GET', '/secure/resource', array(), array(), array(
             'PHP_AUTH_USER' => 'unknown',
@@ -35,7 +34,6 @@ class FirewallEntryPointTest extends WebTestCase
     public function testItUsesTheConfiguredEntryPointFromTheExceptionListenerWithFormLoginAndNoCredentials()
     {
         $client = $this->createClient(array('test_case' => 'FirewallEntryPoint', 'root_config' => 'config_form_login.yml'));
-        $client->insulate();
 
         $client->request('GET', '/secure/resource');
 
@@ -45,18 +43,4 @@ class FirewallEntryPointTest extends WebTestCase
             "Custom entry point wasn't started"
         );
     }
-
-    protected function setUp()
-    {
-        parent::setUp();
-
-        $this->deleteTmpDir('FirewallEntryPoint');
-    }
-
-    protected function tearDown()
-    {
-        parent::tearDown();
-
-        $this->deleteTmpDir('FirewallEntryPoint');
-    }
 }
