@@ -11,6 +11,12 @@
 
 namespace Symfony\Bundle\TwigBundle\DependencyInjection\Configurator;
 
+use Symfony\Bridge\Twig\UndefinedCallableHandler;
+use Twig\Environment;
+
+// BC/FC with namespaced Twig
+class_exists('Twig\Environment');
+
 /**
  * Twig environment configurator.
  *
@@ -35,14 +41,18 @@ class EnvironmentConfigurator
         $this->thousandsSeparator = $thousandsSeparator;
     }
 
-    public function configure(\Twig_Environment $environment)
+    public function configure(Environment $environment)
     {
-        $environment->getExtension('core')->setDateFormat($this->dateFormat, $this->intervalFormat);
+        $environment->getExtension('Twig\Extension\CoreExtension')->setDateFormat($this->dateFormat, $this->intervalFormat);
 
         if (null !== $this->timezone) {
-            $environment->getExtension('core')->setTimezone($this->timezone);
+            $environment->getExtension('Twig\Extension\CoreExtension')->setTimezone($this->timezone);
         }
 
-        $environment->getExtension('core')->setNumberFormat($this->decimals, $this->decimalPoint, $this->thousandsSeparator);
+        $environment->getExtension('Twig\Extension\CoreExtension')->setNumberFormat($this->decimals, $this->decimalPoint, $this->thousandsSeparator);
+
+        // wrap UndefinedCallableHandler in closures for lazy-autoloading
+        $environment->registerUndefinedFilterCallback(function ($name) { return UndefinedCallableHandler::onUndefinedFilter($name); });
+        $environment->registerUndefinedFunctionCallback(function ($name) { return UndefinedCallableHandler::onUndefinedFunction($name); });
     }
 }
