@@ -11,41 +11,33 @@
 
 namespace Symfony\Component\DependencyInjection\Tests;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\DependencyInjection\Definition;
 
-class DefinitionTest extends \PHPUnit_Framework_TestCase
+class DefinitionTest extends TestCase
 {
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::__construct
-     */
     public function testConstructor()
     {
         $def = new Definition('stdClass');
         $this->assertEquals('stdClass', $def->getClass(), '__construct() takes the class name as its first argument');
+        $this->assertSame(array('class' => true), $def->getChanges());
 
         $def = new Definition('stdClass', array('foo'));
         $this->assertEquals(array('foo'), $def->getArguments(), '__construct() takes an optional array of arguments as its second argument');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setFactory
-     * @covers Symfony\Component\DependencyInjection\Definition::getFactory
-     */
     public function testSetGetFactory()
     {
-        $def = new Definition('stdClass');
+        $def = new Definition();
 
         $this->assertSame($def, $def->setFactory('foo'), '->setFactory() implements a fluent interface');
         $this->assertEquals('foo', $def->getFactory(), '->getFactory() returns the factory');
 
         $def->setFactory('Foo::bar');
         $this->assertEquals(array('Foo', 'bar'), $def->getFactory(), '->setFactory() converts string static method call to the array');
+        $this->assertSame(array('factory' => true), $def->getChanges());
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setClass
-     * @covers Symfony\Component\DependencyInjection\Definition::getClass
-     */
     public function testSetGetClass()
     {
         $def = new Definition('stdClass');
@@ -55,29 +47,38 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
 
     public function testSetGetDecoratedService()
     {
+        $def = new Definition('stdClass');
+        $this->assertNull($def->getDecoratedService());
+        $def->setDecoratedService('foo', 'foo.renamed', 5);
+        $this->assertEquals(array('foo', 'foo.renamed', 5), $def->getDecoratedService());
+        $def->setDecoratedService(null);
+        $this->assertNull($def->getDecoratedService());
+
         $def = new Definition('stdClass');
         $this->assertNull($def->getDecoratedService());
         $def->setDecoratedService('foo', 'foo.renamed');
-        $this->assertEquals(array('foo', 'foo.renamed'), $def->getDecoratedService());
+        $this->assertEquals(array('foo', 'foo.renamed', 0), $def->getDecoratedService());
         $def->setDecoratedService(null);
         $this->assertNull($def->getDecoratedService());
 
         $def = new Definition('stdClass');
         $def->setDecoratedService('foo');
-        $this->assertEquals(array('foo', null), $def->getDecoratedService());
+        $this->assertEquals(array('foo', null, 0), $def->getDecoratedService());
         $def->setDecoratedService(null);
         $this->assertNull($def->getDecoratedService());
 
         $def = new Definition('stdClass');
-        $this->setExpectedException('InvalidArgumentException', 'The decorated service inner name for "foo" must be different than the service name itself.');
+
+        if (method_exists($this, 'expectException')) {
+            $this->expectException('InvalidArgumentException');
+            $this->expectExceptionMessage('The decorated service inner name for "foo" must be different than the service name itself.');
+        } else {
+            $this->setExpectedException('InvalidArgumentException', 'The decorated service inner name for "foo" must be different than the service name itself.');
+        }
+
         $def->setDecoratedService('foo', 'foo');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setArguments
-     * @covers Symfony\Component\DependencyInjection\Definition::getArguments
-     * @covers Symfony\Component\DependencyInjection\Definition::addArgument
-     */
     public function testArguments()
     {
         $def = new Definition('stdClass');
@@ -87,12 +88,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals(array('foo', 'bar'), $def->getArguments(), '->addArgument() adds an argument');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setMethodCalls
-     * @covers Symfony\Component\DependencyInjection\Definition::addMethodCall
-     * @covers Symfony\Component\DependencyInjection\Definition::hasMethodCall
-     * @covers Symfony\Component\DependencyInjection\Definition::removeMethodCall
-     */
     public function testMethodCalls()
     {
         $def = new Definition('stdClass');
@@ -116,10 +111,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $def->addMethodCall('');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setFile
-     * @covers Symfony\Component\DependencyInjection\Definition::getFile
-     */
     public function testSetGetFile()
     {
         $def = new Definition('stdClass');
@@ -127,22 +118,14 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('foo', $def->getFile(), '->getFile() returns the file to include');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setScope
-     * @covers Symfony\Component\DependencyInjection\Definition::getScope
-     */
-    public function testSetGetScope()
+    public function testSetIsShared()
     {
         $def = new Definition('stdClass');
-        $this->assertEquals('container', $def->getScope());
-        $this->assertSame($def, $def->setScope('foo'));
-        $this->assertEquals('foo', $def->getScope());
+        $this->assertTrue($def->isShared(), '->isShared() returns true by default');
+        $this->assertSame($def, $def->setShared(false), '->setShared() implements a fluent interface');
+        $this->assertFalse($def->isShared(), '->isShared() returns false if the instance must not be shared');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setPublic
-     * @covers Symfony\Component\DependencyInjection\Definition::isPublic
-     */
     public function testSetIsPublic()
     {
         $def = new Definition('stdClass');
@@ -151,10 +134,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertFalse($def->isPublic(), '->isPublic() returns false if the instance must not be public.');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setSynthetic
-     * @covers Symfony\Component\DependencyInjection\Definition::isSynthetic
-     */
     public function testSetIsSynthetic()
     {
         $def = new Definition('stdClass');
@@ -163,25 +142,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($def->isSynthetic(), '->isSynthetic() returns true if the service is synthetic.');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setSynchronized
-     * @covers Symfony\Component\DependencyInjection\Definition::isSynchronized
-     * @group legacy
-     */
-    public function testLegacySetIsSynchronized()
-    {
-        $this->iniSet('error_reporting', -1 & ~E_USER_DEPRECATED);
-
-        $def = new Definition('stdClass');
-        $this->assertFalse($def->isSynchronized(), '->isSynchronized() returns false by default');
-        $this->assertSame($def, $def->setSynchronized(true), '->setSynchronized() implements a fluent interface');
-        $this->assertTrue($def->isSynchronized(), '->isSynchronized() returns true if the service is synchronized.');
-    }
-
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setLazy
-     * @covers Symfony\Component\DependencyInjection\Definition::isLazy
-     */
     public function testSetIsLazy()
     {
         $def = new Definition('stdClass');
@@ -190,10 +150,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($def->isLazy(), '->isLazy() returns true if the service is lazy.');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setAbstract
-     * @covers Symfony\Component\DependencyInjection\Definition::isAbstract
-     */
     public function testSetIsAbstract()
     {
         $def = new Definition('stdClass');
@@ -202,10 +158,35 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($def->isAbstract(), '->isAbstract() returns true if the instance must not be public.');
     }
 
+    public function testSetIsDeprecated()
+    {
+        $def = new Definition('stdClass');
+        $this->assertFalse($def->isDeprecated(), '->isDeprecated() returns false by default');
+        $this->assertSame($def, $def->setDeprecated(true), '->setDeprecated() implements a fluent interface');
+        $this->assertTrue($def->isDeprecated(), '->isDeprecated() returns true if the instance should not be used anymore.');
+        $this->assertSame('The "deprecated_service" service is deprecated. You should stop using it, as it will soon be removed.', $def->getDeprecationMessage('deprecated_service'), '->getDeprecationMessage() should return a formatted message template');
+    }
+
     /**
-     * @covers Symfony\Component\DependencyInjection\Definition::setConfigurator
-     * @covers Symfony\Component\DependencyInjection\Definition::getConfigurator
+     * @dataProvider invalidDeprecationMessageProvider
+     * @expectedException \Symfony\Component\DependencyInjection\Exception\InvalidArgumentException
      */
+    public function testSetDeprecatedWithInvalidDeprecationTemplate($message)
+    {
+        $def = new Definition('stdClass');
+        $def->setDeprecated(false, $message);
+    }
+
+    public function invalidDeprecationMessageProvider()
+    {
+        return array(
+            "With \rs" => array("invalid \r message %service_id%"),
+            "With \ns" => array("invalid \n message %service_id%"),
+            'With */s' => array('invalid */ message %service_id%'),
+            'message not containing require %service_id% variable' => array('this is deprecated'),
+        );
+    }
+
     public function testSetGetConfigurator()
     {
         $def = new Definition('stdClass');
@@ -213,9 +194,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('foo', $def->getConfigurator(), '->getConfigurator() returns the configurator');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::clearTags
-     */
     public function testClearTags()
     {
         $def = new Definition('stdClass');
@@ -225,9 +203,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals(array(), $def->getTags(), '->clearTags() removes all current tags');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::clearTags
-     */
     public function testClearTag()
     {
         $def = new Definition('stdClass');
@@ -244,12 +219,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($def->hasTag('3foo3'));
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::addTag
-     * @covers Symfony\Component\DependencyInjection\Definition::getTag
-     * @covers Symfony\Component\DependencyInjection\Definition::getTags
-     * @covers Symfony\Component\DependencyInjection\Definition::hasTag
-     */
     public function testTags()
     {
         $def = new Definition('stdClass');
@@ -267,9 +236,6 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         ), '->getTags() returns all tags');
     }
 
-    /**
-     * @covers Symfony\Component\DependencyInjection\Definition::replaceArgument
-     */
     public function testSetArgument()
     {
         $def = new Definition('stdClass');
@@ -301,6 +267,7 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
 
     /**
      * @expectedException \OutOfBoundsException
+     * @expectedExceptionMessage The index "1" is not in the range [0, 0].
      */
     public function testReplaceArgumentShouldCheckBounds()
     {
@@ -310,6 +277,16 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $def->replaceArgument(1, 'bar');
     }
 
+    /**
+     * @expectedException \OutOfBoundsException
+     * @expectedExceptionMessage Cannot replace arguments if none have been configured yet.
+     */
+    public function testReplaceArgumentWithoutExistingArgumentsShouldCheckBounds()
+    {
+        $def = new Definition('stdClass');
+        $def->replaceArgument(0, 'bar');
+    }
+
     public function testSetGetProperties()
     {
         $def = new Definition('stdClass');
@@ -327,4 +304,96 @@ class DefinitionTest extends \PHPUnit_Framework_TestCase
         $this->assertSame($def, $def->setProperty('foo', 'bar'));
         $this->assertEquals(array('foo' => 'bar'), $def->getProperties());
     }
+
+    public function testAutowired()
+    {
+        $def = new Definition('stdClass');
+        $this->assertFalse($def->isAutowired());
+
+        $def->setAutowired(true);
+        $this->assertTrue($def->isAutowired());
+
+        $def->setAutowired(false);
+        $this->assertFalse($def->isAutowired());
+    }
+
+    public function testChangesNoChanges()
+    {
+        $def = new Definition();
+
+        $this->assertSame(array(), $def->getChanges());
+    }
+
+    public function testGetChangesWithChanges()
+    {
+        $def = new Definition('stdClass', array('fooarg'));
+
+        $def->setAbstract(true);
+        $def->setAutowired(true);
+        $def->setConfigurator('configuration_func');
+        $def->setDecoratedService(null);
+        $def->setDeprecated(true);
+        $def->setFactory('factory_func');
+        $def->setFile('foo.php');
+        $def->setLazy(true);
+        $def->setPublic(true);
+        $def->setShared(true);
+        $def->setSynthetic(true);
+        // changes aren't tracked for these, class or arguments
+        $def->setInstanceofConditionals(array());
+        $def->addTag('foo_tag');
+        $def->addMethodCall('methodCall');
+        $def->setProperty('fooprop', true);
+        $def->setAutoconfigured(true);
+
+        $this->assertSame(array(
+            'class' => true,
+            'autowired' => true,
+            'configurator' => true,
+            'decorated_service' => true,
+            'deprecated' => true,
+            'factory' => true,
+            'file' => true,
+            'lazy' => true,
+            'public' => true,
+            'shared' => true,
+            'autoconfigured' => true,
+        ), $def->getChanges());
+
+        $def->setChanges(array());
+        $this->assertSame(array(), $def->getChanges());
+    }
+
+    /**
+     * @group legacy
+     */
+    public function testTypes()
+    {
+        $def = new Definition('stdClass');
+
+        $this->assertEquals(array(), $def->getAutowiringTypes());
+        $this->assertSame($def, $def->setAutowiringTypes(array('Foo')));
+        $this->assertEquals(array('Foo'), $def->getAutowiringTypes());
+        $this->assertSame($def, $def->addAutowiringType('Bar'));
+        $this->assertTrue($def->hasAutowiringType('Bar'));
+        $this->assertSame($def, $def->removeAutowiringType('Foo'));
+        $this->assertEquals(array('Bar'), $def->getAutowiringTypes());
+    }
+
+    public function testShouldAutoconfigure()
+    {
+        $def = new Definition('stdClass');
+        $this->assertFalse($def->isAutoconfigured());
+        $def->setAutoconfigured(true);
+        $this->assertTrue($def->isAutoconfigured());
+    }
+
+    public function testAddError()
+    {
+        $def = new Definition('stdClass');
+        $this->assertEmpty($def->getErrors());
+        $def->addError('First error');
+        $def->addError('Second error');
+        $this->assertSame(array('First error', 'Second error'), $def->getErrors());
+    }
 }
