@@ -1,10 +1,17 @@
 <?php
 
+namespace Bar;
+
 class FooClass
 {
-    public $foo, $moo;
+    public $foo;
+    public $moo;
 
-    public $bar = null, $initialized = false, $configured = false, $called = false, $arguments = array();
+    public $bar = null;
+    public $initialized = false;
+    public $configured = false;
+    public $called = false;
+    public $arguments = array();
 
     public function __construct($arguments = array())
     {
