@@ -11,24 +11,23 @@
 
 namespace Symfony\Component\DependencyInjection\Tests\LazyProxy\Instantiator;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\DependencyInjection\Definition;
 use Symfony\Component\DependencyInjection\LazyProxy\Instantiator\RealServiceInstantiator;
 
 /**
- * Tests for {@see \Symfony\Component\DependencyInjection\Instantiator\RealServiceInstantiator}
+ * Tests for {@see \Symfony\Component\DependencyInjection\Instantiator\RealServiceInstantiator}.
  *
  * @author Marco Pivetta <ocramius@gmail.com>
- *
- * @covers \Symfony\Component\DependencyInjection\LazyProxy\Instantiator\RealServiceInstantiator
  */
-class RealServiceInstantiatorTest extends \PHPUnit_Framework_TestCase
+class RealServiceInstantiatorTest extends TestCase
 {
     public function testInstantiateProxy()
     {
         $instantiator = new RealServiceInstantiator();
-        $instance     = new \stdClass();
-        $container    = $this->getMock('Symfony\Component\DependencyInjection\ContainerInterface');
-        $callback     = function () use ($instance) {
+        $instance = new \stdClass();
+        $container = $this->getMockBuilder('Symfony\Component\DependencyInjection\ContainerInterface')->getMock();
+        $callback = function () use ($instance) {
             return $instance;
         };
 
