@@ -55,18 +55,19 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Creates a new button builder.
      *
-     * @param string $name    The name of the button.
-     * @param array  $options The button's options.
+     * @param string $name    The name of the button
+     * @param array  $options The button's options
      *
-     * @throws InvalidArgumentException If the name is empty.
+     * @throws InvalidArgumentException if the name is empty
      */
     public function __construct($name, array $options = array())
     {
-        if (empty($name) && 0 != $name) {
+        $name = (string) $name;
+        if ('' === $name) {
             throw new InvalidArgumentException('Buttons cannot have empty names.');
         }
 
-        $this->name = (string) $name;
+        $this->name = $name;
         $this->options = $options;
     }
 
@@ -135,7 +136,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * @param string $name
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function has($name)
     {
@@ -145,7 +146,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns the children.
      *
-     * @return array Always returns an empty array.
+     * @return array Always returns an empty array
      */
     public function all()
     {
@@ -183,8 +184,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * This method should not be invoked.
      *
-     * @param EventSubscriberInterface $subscriber
-     *
      * @throws BadMethodCallException
      */
     public function addEventSubscriber(EventSubscriberInterface $subscriber)
@@ -271,8 +270,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * This method should not be invoked.
      *
-     * @param DataMapperInterface $dataMapper
-     *
      * @throws BadMethodCallException
      */
     public function setDataMapper(DataMapperInterface $dataMapper = null)
@@ -285,7 +282,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * @param bool $disabled Whether the button is disabled
      *
-     * @return ButtonBuilder The button builder.
+     * @return $this
      */
     public function setDisabled($disabled)
     {
@@ -378,20 +375,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
         throw new BadMethodCallException('Buttons do not support data mapping.');
     }
 
-    /**
-     * Unsupported method.
-     *
-     * This method should not be invoked.
-     *
-     * @param bool $virtual
-     *
-     * @throws BadMethodCallException
-     */
-    public function setVirtual($virtual)
-    {
-        throw new BadMethodCallException('Buttons cannot be virtual.');
-    }
-
     /**
      * Unsupported method.
      *
@@ -409,9 +392,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Sets the type of the button.
      *
-     * @param ResolvedFormTypeInterface $type The type of the button.
-     *
-     * @return ButtonBuilder The button builder.
+     * @return $this
      */
     public function setType(ResolvedFormTypeInterface $type)
     {
@@ -453,8 +434,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * This method should not be invoked.
      *
-     * @param FormFactoryInterface $formFactory
-     *
      * @throws BadMethodCallException
      */
     public function setFormFactory(FormFactoryInterface $formFactory)
@@ -489,13 +468,11 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @param RequestHandlerInterface $requestHandler
-     *
      * @throws BadMethodCallException
      */
     public function setRequestHandler(RequestHandlerInterface $requestHandler)
     {
-        throw new BadMethodCallException('Buttons do not support form processors.');
+        throw new BadMethodCallException('Buttons do not support request handlers.');
     }
 
     /**
@@ -503,7 +480,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * @param bool $initialize
      *
-     * @return ButtonBuilder
+     * @return $this
      *
      * @throws BadMethodCallException
      */
@@ -544,8 +521,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getEventDispatcher()
     {
@@ -561,8 +536,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getPropertyPath()
     {
@@ -571,7 +544,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getMapped()
     {
@@ -581,7 +554,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getByReference()
     {
@@ -591,17 +564,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
-     */
-    public function getVirtual()
-    {
-        return false;
-    }
-
-    /**
-     * Unsupported method.
-     *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getCompound()
     {
@@ -611,7 +574,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns the form type used to construct the button.
      *
-     * @return ResolvedFormTypeInterface The button's type.
+     * @return ResolvedFormTypeInterface The button's type
      */
     public function getType()
     {
@@ -621,7 +584,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return array Always returns an empty array.
+     * @return array Always returns an empty array
      */
     public function getViewTransformers()
     {
@@ -631,7 +594,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return array Always returns an empty array.
+     * @return array Always returns an empty array
      */
     public function getModelTransformers()
     {
@@ -640,8 +603,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getDataMapper()
     {
@@ -650,7 +611,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getRequired()
     {
@@ -660,7 +621,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns whether the button is disabled.
      *
-     * @return bool Whether the button is disabled.
+     * @return bool Whether the button is disabled
      */
     public function getDisabled()
     {
@@ -670,7 +631,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getErrorBubbling()
     {
@@ -679,8 +640,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getEmptyData()
     {
@@ -689,7 +648,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns additional attributes of the button.
      *
-     * @return array An array of key-value combinations.
+     * @return array An array of key-value combinations
      */
     public function getAttributes()
     {
@@ -699,9 +658,9 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns whether the attribute with the given name exists.
      *
-     * @param string $name The attribute name.
+     * @param string $name The attribute name
      *
-     * @return bool Whether the attribute exists.
+     * @return bool Whether the attribute exists
      */
     public function hasAttribute($name)
     {
@@ -711,10 +670,10 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns the value of the given attribute.
      *
-     * @param string $name    The attribute name.
-     * @param mixed  $default The value returned if the attribute does not exist.
+     * @param string $name    The attribute name
+     * @param mixed  $default The value returned if the attribute does not exist
      *
-     * @return mixed The attribute value.
+     * @return mixed The attribute value
      */
     public function getAttribute($name, $default = null)
     {
@@ -723,8 +682,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getData()
     {
@@ -732,8 +689,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getDataClass()
     {
@@ -742,7 +697,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getDataLocked()
     {
@@ -751,8 +706,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getFormFactory()
     {
@@ -760,8 +713,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getAction()
     {
@@ -769,8 +720,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getMethod()
     {
@@ -778,8 +727,6 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
 
     /**
      * Unsupported method.
-     *
-     * @return null Always returns null.
      */
     public function getRequestHandler()
     {
@@ -788,7 +735,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getAutoInitialize()
     {
@@ -798,7 +745,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return bool Always returns false.
+     * @return bool Always returns false
      */
     public function getInheritData()
     {
@@ -808,7 +755,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns all options passed during the construction of the button.
      *
-     * @return array The passed options.
+     * @return array The passed options
      */
     public function getOptions()
     {
@@ -820,7 +767,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
      *
      * @param string $name The option name,
      *
-     * @return bool Whether the option exists.
+     * @return bool Whether the option exists
      */
     public function hasOption($name)
     {
@@ -830,10 +777,10 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Returns the value of a specific option.
      *
-     * @param string $name    The option name.
-     * @param mixed  $default The value returned if the option does not exist.
+     * @param string $name    The option name
+     * @param mixed  $default The value returned if the option does not exist
      *
-     * @return mixed The option value.
+     * @return mixed The option value
      */
     public function getOption($name, $default = null)
     {
@@ -843,7 +790,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return int Always returns 0.
+     * @return int Always returns 0
      */
     public function count()
     {
@@ -853,7 +800,7 @@ class ButtonBuilder implements \IteratorAggregate, FormBuilderInterface
     /**
      * Unsupported method.
      *
-     * @return \EmptyIterator Always returns an empty iterator.
+     * @return \EmptyIterator Always returns an empty iterator
      */
     public function getIterator()
     {
