@@ -13,7 +13,7 @@ namespace Symfony\Component\Form\Extension\Core\DataTransformer;
 
 use Symfony\Component\Form\DataTransformerInterface;
 use Symfony\Component\Form\Exception\TransformationFailedException;
-use Symfony\Component\Form\Extension\Core\ChoiceList\ChoiceListInterface;
+use Symfony\Component\Form\ChoiceList\ChoiceListInterface;
 
 /**
  * @author Bernhard Schussek <bschussek@gmail.com>
@@ -22,11 +22,6 @@ class ChoiceToValueTransformer implements DataTransformerInterface
 {
     private $choiceList;
 
-    /**
-     * Constructor.
-     *
-     * @param ChoiceListInterface $choiceList
-     */
     public function __construct(ChoiceListInterface $choiceList)
     {
         $this->choiceList = $choiceList;
@@ -39,24 +34,20 @@ class ChoiceToValueTransformer implements DataTransformerInterface
 
     public function reverseTransform($value)
     {
-        if (null !== $value && !is_scalar($value)) {
-            throw new TransformationFailedException('Expected a scalar.');
+        if (null !== $value && !is_string($value)) {
+            throw new TransformationFailedException('Expected a string or null.');
         }
 
-        // These are now valid ChoiceList values, so we can return null
-        // right away
-        if ('' === $value || null === $value) {
-            return;
-        }
-
-        $choices = $this->choiceList->getChoicesForValues(array($value));
+        $choices = $this->choiceList->getChoicesForValues(array((string) $value));
 
         if (1 !== count($choices)) {
+            if (null === $value || '' === $value) {
+                return;
+            }
+
             throw new TransformationFailedException(sprintf('The choice "%s" does not exist or is not unique', $value));
         }
 
-        $choice = current($choices);
-
-        return '' === $choice ? null : $choice;
+        return current($choices);
     }
 }
