@@ -14,7 +14,7 @@ namespace Symfony\Component\Form\Extension\Core\Type;
 use Symfony\Component\Form\AbstractType;
 use Symfony\Component\Form\FormBuilderInterface;
 use Symfony\Component\Form\Extension\Core\EventListener\FixUrlProtocolListener;
-use Symfony\Component\OptionsResolver\OptionsResolverInterface;
+use Symfony\Component\OptionsResolver\OptionsResolver;
 
 class UrlType extends AbstractType
 {
@@ -31,15 +31,11 @@ class UrlType extends AbstractType
     /**
      * {@inheritdoc}
      */
-    public function setDefaultOptions(OptionsResolverInterface $resolver)
+    public function configureOptions(OptionsResolver $resolver)
     {
-        $resolver->setDefaults(array(
-            'default_protocol' => 'http',
-        ));
+        $resolver->setDefault('default_protocol', 'http');
 
-        $resolver->setAllowedTypes(array(
-            'default_protocol' => array('null', 'string'),
-        ));
+        $resolver->setAllowedTypes('default_protocol', array('null', 'string'));
     }
 
     /**
@@ -47,13 +43,13 @@ class UrlType extends AbstractType
      */
     public function getParent()
     {
-        return 'text';
+        return __NAMESPACE__.'\TextType';
     }
 
     /**
      * {@inheritdoc}
      */
-    public function getName()
+    public function getBlockPrefix()
     {
         return 'url';
     }
