@@ -11,22 +11,16 @@
 
 namespace Symfony\Component\Form\Extension\DataCollector;
 
-use Symfony\Component\EventDispatcher\EventSubscriberInterface;
 use Symfony\Component\Form\AbstractExtension;
 
 /**
  * Extension for collecting data of the forms on a page.
  *
- * @since  2.4
- *
  * @author Robert Schönthal <robert.schoenthal@gmail.com>
  * @author Bernhard Schussek <bschussek@gmail.com>
  */
 class DataCollectorExtension extends AbstractExtension
 {
-    /**
-     * @var EventSubscriberInterface
-     */
     private $dataCollector;
 
     public function __construct(FormDataCollectorInterface $dataCollector)
