@@ -13,7 +13,8 @@ namespace Symfony\Component\Form\Extension\Validator\Type;
 
 use Symfony\Component\Form\AbstractTypeExtension;
 use Symfony\Component\OptionsResolver\Options;
-use Symfony\Component\OptionsResolver\OptionsResolverInterface;
+use Symfony\Component\OptionsResolver\OptionsResolver;
+use Symfony\Component\Validator\Constraints\GroupSequence;
 
 /**
  * Encapsulates common logic of {@link FormTypeValidatorExtension} and
@@ -26,7 +27,7 @@ abstract class BaseValidatorExtension extends AbstractTypeExtension
     /**
      * {@inheritdoc}
      */
-    public function setDefaultOptions(OptionsResolverInterface $resolver)
+    public function configureOptions(OptionsResolver $resolver)
     {
         // Make sure that validation groups end up as null, closure or array
         $validationGroupsNormalizer = function (Options $options, $groups) {
@@ -42,6 +43,10 @@ abstract class BaseValidatorExtension extends AbstractTypeExtension
                 return $groups;
             }
 
+            if ($groups instanceof GroupSequence) {
+                return $groups;
+            }
+
             return (array) $groups;
         };
 
@@ -49,8 +54,6 @@ abstract class BaseValidatorExtension extends AbstractTypeExtension
             'validation_groups' => null,
         ));
 
-        $resolver->setNormalizers(array(
-            'validation_groups' => $validationGroupsNormalizer,
-        ));
+        $resolver->setNormalizer('validation_groups', $validationGroupsNormalizer);
     }
 }
