@@ -12,6 +12,7 @@
 namespace Symfony\Component\Form;
 
 use Symfony\Component\Form\Exception\ExceptionInterface;
+use Symfony\Component\Form\Exception\LogicException;
 use Symfony\Component\Form\Exception\UnexpectedTypeException;
 use Symfony\Component\Form\Exception\InvalidArgumentException;
 
@@ -34,11 +35,6 @@ class FormRegistry implements FormRegistryInterface
      */
     private $types = array();
 
-    /**
-     * @var string[]
-     */
-    private $legacyNames = array();
-
     /**
      * @var FormTypeGuesserInterface|false|null
      */
@@ -49,11 +45,11 @@ class FormRegistry implements FormRegistryInterface
      */
     private $resolvedTypeFactory;
 
+    private $checkedTypes = array();
+
     /**
-     * Constructor.
-     *
      * @param FormExtensionInterface[]         $extensions          An array of FormExtensionInterface
-     * @param ResolvedFormTypeFactoryInterface $resolvedTypeFactory The factory for resolved form types.
+     * @param ResolvedFormTypeFactoryInterface $resolvedTypeFactory The factory for resolved form types
      *
      * @throws UnexpectedTypeException if any extension does not implement FormExtensionInterface
      */
@@ -86,18 +82,17 @@ class FormRegistry implements FormRegistryInterface
 
             if (!$type) {
                 // Support fully-qualified class names
-                if (class_exists($name) && in_array('Symfony\Component\Form\FormTypeInterface', class_implements($name))) {
-                    $type = new $name();
-                } else {
-                    throw new InvalidArgumentException(sprintf('Could not load type "%s"', $name));
+                if (!class_exists($name)) {
+                    throw new InvalidArgumentException(sprintf('Could not load type "%s": class does not exist.', $name));
                 }
+                if (!is_subclass_of($name, 'Symfony\Component\Form\FormTypeInterface')) {
+                    throw new InvalidArgumentException(sprintf('Could not load type "%s": class does not implement "Symfony\Component\Form\FormTypeInterface".', $name));
+                }
+
+                $type = new $name();
             }
 
-            $this->resolveAndAddType($type);
-        }
-
-        if (isset($this->legacyNames[$name])) {
-            @trigger_error(sprintf('Accessing type "%s" by its string name is deprecated since version 2.8 and will be removed in 3.0. Use the fully-qualified type class name "%s" instead.', $name, get_class($this->types[$name]->getInnerType())), E_USER_DEPRECATED);
+            $this->types[$name] = $this->resolveType($type);
         }
 
         return $this->types[$name];
@@ -107,54 +102,38 @@ class FormRegistry implements FormRegistryInterface
      * Wraps a type into a ResolvedFormTypeInterface implementation and connects
      * it with its parent type.
      *
-     * @param FormTypeInterface $type The type to resolve.
+     * @param FormTypeInterface $type The type to resolve
      *
-     * @return ResolvedFormTypeInterface The resolved type.
+     * @return ResolvedFormTypeInterface The resolved type
      */
-    private function resolveAndAddType(FormTypeInterface $type)
+    private function resolveType(FormTypeInterface $type)
     {
         $typeExtensions = array();
         $parentType = $type->getParent();
         $fqcn = get_class($type);
-        $name = $type->getName();
-        $hasCustomName = $name !== $fqcn;
 
-        if ($parentType instanceof FormTypeInterface) {
-            @trigger_error(sprintf('Returning a FormTypeInterface from %s::getParent() is deprecated since version 2.8 and will be removed in 3.0. Return the fully-qualified type class name instead.', $fqcn), E_USER_DEPRECATED);
-
-            $this->resolveAndAddType($parentType);
-            $parentType = $parentType->getName();
+        if (isset($this->checkedTypes[$fqcn])) {
+            $types = implode(' > ', array_merge(array_keys($this->checkedTypes), array($fqcn)));
+            throw new LogicException(sprintf('Circular reference detected for form type "%s" (%s).', $fqcn, $types));
         }
 
-        if ($hasCustomName) {
+        $this->checkedTypes[$fqcn] = true;
+
+        try {
             foreach ($this->extensions as $extension) {
-                if ($x = $extension->getTypeExtensions($name)) {
-                    @trigger_error(sprintf('Returning a type name from %s::getExtendedType() is deprecated since version 2.8 and will be removed in 3.0. Return the fully-qualified type class name instead.', get_class($x[0])), E_USER_DEPRECATED);
-
-                    $typeExtensions = array_merge($typeExtensions, $x);
-                }
+                $typeExtensions = array_merge(
+                    $typeExtensions,
+                    $extension->getTypeExtensions($fqcn)
+                );
             }
-        }
 
-        foreach ($this->extensions as $extension) {
-            $typeExtensions = array_merge(
+            return $this->resolvedTypeFactory->createResolvedType(
+                $type,
                 $typeExtensions,
-                $extension->getTypeExtensions($fqcn)
+                $parentType ? $this->getType($parentType) : null
             );
-        }
-
-        $resolvedType = $this->resolvedTypeFactory->createResolvedType(
-            $type,
-            $typeExtensions,
-            $parentType ? $this->getType($parentType) : null
-        );
-
-        $this->types[$fqcn] = $resolvedType;
-
-        if ($hasCustomName) {
-            // Enable access by the explicit type name until Symfony 3.0
-            $this->types[$name] = $resolvedType;
-            $this->legacyNames[$name] = true;
+        } finally {
+            unset($this->checkedTypes[$fqcn]);
         }
     }
 
@@ -163,10 +142,6 @@ class FormRegistry implements FormRegistryInterface
      */
     public function hasType($name)
     {
-        if (isset($this->legacyNames[$name])) {
-            @trigger_error(sprintf('Accessing type "%s" by its string name is deprecated since version 2.8 and will be removed in 3.0. Use the fully-qualified type class name "%s" instead.', $name, get_class($this->types[$name]->getInnerType())), E_USER_DEPRECATED);
-        }
-
         if (isset($this->types[$name])) {
             return true;
         }
