@@ -11,13 +11,14 @@
 
 namespace Symfony\Component\Form\Test;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\Form\Forms;
 use Symfony\Component\Form\FormFactoryInterface;
 
 /**
  * @author Bernhard Schussek <bschussek@gmail.com>
  */
-abstract class FormIntegrationTestCase extends \PHPUnit_Framework_TestCase
+abstract class FormIntegrationTestCase extends TestCase
 {
     /**
      * @var FormFactoryInterface
@@ -28,6 +29,9 @@ abstract class FormIntegrationTestCase extends \PHPUnit_Framework_TestCase
     {
         $this->factory = Forms::createFormFactoryBuilder()
             ->addExtensions($this->getExtensions())
+            ->addTypeExtensions($this->getTypeExtensions())
+            ->addTypes($this->getTypes())
+            ->addTypeGuessers($this->getTypeGuessers())
             ->getFormFactory();
     }
 
@@ -35,4 +39,19 @@ abstract class FormIntegrationTestCase extends \PHPUnit_Framework_TestCase
     {
         return array();
     }
+
+    protected function getTypeExtensions()
+    {
+        return array();
+    }
+
+    protected function getTypes()
+    {
+        return array();
+    }
+
+    protected function getTypeGuessers()
+    {
+        return array();
+    }
 }
