@@ -298,6 +298,8 @@ class CompoundFormTest extends AbstractFormTest
     public function testRemoveIgnoresUnknownName()
     {
         $this->form->remove('notexisting');
+
+        $this->assertCount(0, $this->form);
     }
 
     public function testArrayAccess()
@@ -306,12 +308,12 @@ class CompoundFormTest extends AbstractFormTest
 
         $this->form[] = $child;
 
-        $this->assertTrue(isset($this->form['foo']));
+        $this->assertArrayHasKey('foo', $this->form);
         $this->assertSame($child, $this->form['foo']);
 
         unset($this->form['foo']);
 
-        $this->assertFalse(isset($this->form['foo']));
+        $this->assertArrayNotHasKey('foo', $this->form);
     }
 
     public function testCountable()
@@ -332,7 +334,6 @@ class CompoundFormTest extends AbstractFormTest
 
     public function testAddMapsViewDataToFormIfInitialized()
     {
-        $test = $this;
         $mapper = $this->getDataMapper();
         $form = $this->getBuilder()
             ->setCompound(true)
@@ -348,9 +349,9 @@ class CompoundFormTest extends AbstractFormTest
         $mapper->expects($this->once())
             ->method('mapDataToForms')
             ->with('bar', $this->isInstanceOf('\RecursiveIteratorIterator'))
-            ->will($this->returnCallback(function ($data, \RecursiveIteratorIterator $iterator) use ($child, $test) {
-                $test->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
-                $test->assertSame(array($child->getName() => $child), iterator_to_array($iterator));
+            ->will($this->returnCallback(function ($data, \RecursiveIteratorIterator $iterator) use ($child) {
+                $this->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
+                $this->assertSame(array($child->getName() => $child), iterator_to_array($iterator));
             }));
 
         $form->initialize();
@@ -426,7 +427,6 @@ class CompoundFormTest extends AbstractFormTest
 
     public function testSetDataMapsViewDataToChildren()
     {
-        $test = $this;
         $mapper = $this->getDataMapper();
         $form = $this->getBuilder()
             ->setCompound(true)
@@ -443,9 +443,9 @@ class CompoundFormTest extends AbstractFormTest
         $mapper->expects($this->once())
             ->method('mapDataToForms')
             ->with('bar', $this->isInstanceOf('\RecursiveIteratorIterator'))
-            ->will($this->returnCallback(function ($data, \RecursiveIteratorIterator $iterator) use ($child1, $child2, $test) {
-                $test->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
-                $test->assertSame(array('firstName' => $child1, 'lastName' => $child2), iterator_to_array($iterator));
+            ->will($this->returnCallback(function ($data, \RecursiveIteratorIterator $iterator) use ($child1, $child2) {
+                $this->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
+                $this->assertSame(array('firstName' => $child1, 'lastName' => $child2), iterator_to_array($iterator));
             }));
 
         $form->setData('foo');
@@ -481,7 +481,6 @@ class CompoundFormTest extends AbstractFormTest
 
     public function testSubmitMapsSubmittedChildrenOntoExistingViewData()
     {
-        $test = $this;
         $mapper = $this->getDataMapper();
         $form = $this->getBuilder()
             ->setCompound(true)
@@ -499,11 +498,11 @@ class CompoundFormTest extends AbstractFormTest
         $mapper->expects($this->once())
             ->method('mapFormsToData')
             ->with($this->isInstanceOf('\RecursiveIteratorIterator'), 'bar')
-            ->will($this->returnCallback(function (\RecursiveIteratorIterator $iterator) use ($child1, $child2, $test) {
-                $test->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
-                $test->assertSame(array('firstName' => $child1, 'lastName' => $child2), iterator_to_array($iterator));
-                $test->assertEquals('Bernhard', $child1->getData());
-                $test->assertEquals('Schussek', $child2->getData());
+            ->will($this->returnCallback(function (\RecursiveIteratorIterator $iterator) use ($child1, $child2) {
+                $this->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
+                $this->assertSame(array('firstName' => $child1, 'lastName' => $child2), iterator_to_array($iterator));
+                $this->assertEquals('Bernhard', $child1->getData());
+                $this->assertEquals('Schussek', $child2->getData());
             }));
 
         $form->submit(array(
@@ -557,7 +556,6 @@ class CompoundFormTest extends AbstractFormTest
 
     public function testSubmitMapsSubmittedChildrenOntoEmptyData()
     {
-        $test = $this;
         $mapper = $this->getDataMapper();
         $object = new \stdClass();
         $form = $this->getBuilder()
@@ -572,9 +570,9 @@ class CompoundFormTest extends AbstractFormTest
         $mapper->expects($this->once())
             ->method('mapFormsToData')
             ->with($this->isInstanceOf('\RecursiveIteratorIterator'), $object)
-            ->will($this->returnCallback(function (\RecursiveIteratorIterator $iterator) use ($child, $test) {
-                $test->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
-                $test->assertSame(array('name' => $child), iterator_to_array($iterator));
+            ->will($this->returnCallback(function (\RecursiveIteratorIterator $iterator) use ($child) {
+                $this->assertInstanceOf('Symfony\Component\Form\Util\InheritDataAwareIterator', $iterator->getInnerIterator());
+                $this->assertSame(array('name' => $child), iterator_to_array($iterator));
             }));
 
         $form->submit(array(
@@ -807,50 +805,6 @@ class CompoundFormTest extends AbstractFormTest
         $this->assertEquals(array('extra' => 'data'), $form->getExtraData());
     }
 
-    /**
-     * @group legacy
-     */
-    public function testGetErrorsAsStringDeep()
-    {
-        $parent = $this->getBuilder()
-            ->setCompound(true)
-            ->setDataMapper($this->getDataMapper())
-            ->getForm();
-
-        $this->form->addError(new FormError('Error!'));
-
-        $parent->add($this->form);
-        $parent->add($this->getBuilder('foo')->getForm());
-
-        $this->assertSame(
-             "name:\n".
-             "    ERROR: Error!\n",
-             $parent->getErrorsAsString()
-        );
-    }
-
-    /**
-     * @group legacy
-     */
-    public function testGetErrorsAsStringDeepWithIndentation()
-    {
-        $parent = $this->getBuilder()
-            ->setCompound(true)
-            ->setDataMapper($this->getDataMapper())
-            ->getForm();
-
-        $this->form->addError(new FormError('Error!'));
-
-        $parent->add($this->form);
-        $parent->add($this->getBuilder('foo')->getForm());
-
-        $this->assertSame(
-             "    name:\n".
-             "        ERROR: Error!\n",
-             $parent->getErrorsAsString(4)
-        );
-    }
-
     public function testGetErrors()
     {
         $this->form->addError($error1 = new FormError('Error 1'));
@@ -925,7 +879,7 @@ class CompoundFormTest extends AbstractFormTest
     // Basic cases are covered in SimpleFormTest
     public function testCreateViewWithChildren()
     {
-        $type = $this->getMock('Symfony\Component\Form\ResolvedFormTypeInterface');
+        $type = $this->getMockBuilder('Symfony\Component\Form\ResolvedFormTypeInterface')->getMock();
         $options = array('a' => 'Foo', 'b' => 'Bar');
         $field1 = $this->getMockForm('foo');
         $field2 = $this->getMockForm('bar');
@@ -941,12 +895,9 @@ class CompoundFormTest extends AbstractFormTest
         $this->form->add($field1);
         $this->form->add($field2);
 
-        $test = $this;
-
-        $assertChildViewsEqual = function (array $childViews) use ($test) {
-            return function (FormView $view) use ($test, $childViews) {
-                /* @var \PHPUnit_Framework_TestCase $test */
-                $test->assertSame($childViews, $view->children);
+        $assertChildViewsEqual = function (array $childViews) {
+            return function (FormView $view) use ($childViews) {
+                $this->assertSame($childViews, $view->children);
             };
         };
 
@@ -1063,6 +1014,28 @@ class CompoundFormTest extends AbstractFormTest
         $this->assertSame($button, $this->form->getClickedButton());
     }
 
+    public function testDisabledButtonIsNotSubmitted()
+    {
+        $button = new SubmitButtonBuilder('submit');
+        $submit = $button
+            ->setDisabled(true)
+            ->getForm();
+
+        $form = $this->createForm()
+            ->add($this->getBuilder('text')->getForm())
+            ->add($submit)
+        ;
+
+        $form->submit(array(
+            'text' => '',
+            'submit' => '',
+        ));
+
+        $this->assertTrue($submit->isDisabled());
+        $this->assertFalse($submit->isClicked());
+        $this->assertFalse($submit->isSubmitted());
+    }
+
     protected function createForm()
     {
         return $this->getBuilder()
