@@ -16,23 +16,10 @@ namespace Symfony\Component\Form\Tests\Extension\Core\Type;
  */
 class ButtonTypeTest extends BaseTypeTest
 {
-    /**
-     * @group legacy
-     */
-    public function testLegacyName()
-    {
-        $form = $this->factory->create('button');
-
-        $this->assertSame('button', $form->getConfig()->getType()->getName());
-    }
+    const TESTED_TYPE = 'Symfony\Component\Form\Extension\Core\Type\ButtonType';
 
     public function testCreateButtonInstances()
     {
-        $this->assertInstanceOf('Symfony\Component\Form\Button', $this->factory->create('Symfony\Component\Form\Extension\Core\Type\ButtonType'));
-    }
-
-    protected function getTestedType()
-    {
-        return 'Symfony\Component\Form\Extension\Core\Type\ButtonType';
+        $this->assertInstanceOf('Symfony\Component\Form\Button', $this->factory->create(static::TESTED_TYPE));
     }
 }
