@@ -49,7 +49,7 @@ class DateTimeTypeTest extends BaseTypeTest
 
         $dateTime = new \DateTime('2010-06-02 03:04:00 UTC');
 
-        $this->assertDateTimeEquals($dateTime, $form->getData());
+        $this->assertEquals($dateTime, $form->getData());
     }
 
     public function testSubmitString()
@@ -133,7 +133,7 @@ class DateTimeTypeTest extends BaseTypeTest
 
         $form->submit($input);
 
-        $this->assertDateTimeEquals(new \DateTime('2010-06-02 03:00:00 UTC'), $form->getData());
+        $this->assertEquals(new \DateTime('2010-06-02 03:00:00 UTC'), $form->getData());
     }
 
     public function testSubmitWithSeconds()
@@ -165,7 +165,7 @@ class DateTimeTypeTest extends BaseTypeTest
 
         $form->submit($input);
 
-        $this->assertDateTimeEquals(new \DateTime('2010-06-02 03:04:05 UTC'), $form->getData());
+        $this->assertEquals(new \DateTime('2010-06-02 03:04:05 UTC'), $form->getData());
     }
 
     public function testSubmitDifferentTimezones()
@@ -215,7 +215,7 @@ class DateTimeTypeTest extends BaseTypeTest
 
         $outputTime->setTimezone(new \DateTimeZone('America/New_York'));
 
-        $this->assertDateTimeEquals($outputTime, $form->getData());
+        $this->assertEquals($outputTime, $form->getData());
         $this->assertEquals('2010-06-02T03:04:00-10:00', $form->getViewData());
     }
 
@@ -266,7 +266,7 @@ class DateTimeTypeTest extends BaseTypeTest
             'time' => '03:04',
         ));
 
-        $this->assertDateTimeEquals($dateTime, $form->getData());
+        $this->assertEquals($dateTime, $form->getData());
     }
 
     public function testInitializeWithDateTime()
