@@ -11,6 +11,7 @@
 
 namespace Symfony\Component\Form\Tests\Extension\Validator\EventListener;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\Form\FormBuilder;
 use Symfony\Component\Form\FormEvent;
 use Symfony\Component\Form\Extension\Validator\Constraints\Form;
@@ -19,7 +20,7 @@ use Symfony\Component\PropertyAccess\PropertyPath;
 use Symfony\Component\Validator\ConstraintViolation;
 use Symfony\Component\Validator\ConstraintViolationList;
 
-class ValidationListenerTest extends \PHPUnit_Framework_TestCase
+class ValidationListenerTest extends TestCase
 {
     /**
      * @var \PHPUnit_Framework_MockObject_MockObject
@@ -54,19 +55,19 @@ class ValidationListenerTest extends \PHPUnit_Framework_TestCase
 
     protected function setUp()
     {
-        $this->dispatcher = $this->getMock('Symfony\Component\EventDispatcher\EventDispatcherInterface');
-        $this->factory = $this->getMock('Symfony\Component\Form\FormFactoryInterface');
-        $this->validator = $this->getMock('Symfony\Component\Validator\Validator\ValidatorInterface');
-        $this->violationMapper = $this->getMock('Symfony\Component\Form\Extension\Validator\ViolationMapper\ViolationMapperInterface');
+        $this->dispatcher = $this->getMockBuilder('Symfony\Component\EventDispatcher\EventDispatcherInterface')->getMock();
+        $this->factory = $this->getMockBuilder('Symfony\Component\Form\FormFactoryInterface')->getMock();
+        $this->validator = $this->getMockBuilder('Symfony\Component\Validator\Validator\ValidatorInterface')->getMock();
+        $this->violationMapper = $this->getMockBuilder('Symfony\Component\Form\Extension\Validator\ViolationMapper\ViolationMapperInterface')->getMock();
         $this->listener = new ValidationListener($this->validator, $this->violationMapper);
         $this->message = 'Message';
         $this->messageTemplate = 'Message template';
         $this->params = array('foo' => 'bar');
     }
 
-    private function getConstraintViolation($code = null, $constraint = null)
+    private function getConstraintViolation($code = null)
     {
-        return new ConstraintViolation($this->message, $this->messageTemplate, $this->params, null, 'prop.path', null, null, $code, $constraint);
+        return new ConstraintViolation($this->message, $this->messageTemplate, $this->params, null, 'prop.path', null, null, $code, new Form());
     }
 
     private function getBuilder($name = 'name', $propertyPath = null, $dataClass = null)
@@ -87,13 +88,13 @@ class ValidationListenerTest extends \PHPUnit_Framework_TestCase
 
     private function getMockForm()
     {
-        return $this->getMock('Symfony\Component\Form\Test\FormInterface');
+        return $this->getMockBuilder('Symfony\Component\Form\Test\FormInterface')->getMock();
     }
 
     // More specific mapping tests can be found in ViolationMapperTest
     public function testMapViolation()
     {
-        $violation = $this->getConstraintViolation(null, new Form());
+        $violation = $this->getConstraintViolation();
         $form = $this->getForm('street');
 
         $this->validator->expects($this->once())
@@ -109,28 +110,7 @@ class ValidationListenerTest extends \PHPUnit_Framework_TestCase
 
     public function testMapViolationAllowsNonSyncIfInvalid()
     {
-        $violation = $this->getConstraintViolation(Form::NOT_SYNCHRONIZED_ERROR, new Form());
-        $form = $this->getForm('street');
-
-        $this->validator->expects($this->once())
-            ->method('validate')
-            ->will($this->returnValue(array($violation)));
-
-        $this->violationMapper->expects($this->once())
-            ->method('mapViolation')
-            // pass true now
-            ->with($violation, $form, true);
-
-        $this->listener->validateForm(new FormEvent($form, null));
-    }
-
-    public function testMapViolationAllowsNonSyncIfInvalidWithoutConstraintReference()
-    {
-        // constraint violations have no reference to the constraint if they are created by
-        //   Symfony\Component\Validator\ExecutionContext
-        // which is deprecated in favor of
-        //   Symfony\Component\Validator\Context\ExecutionContext
-        $violation = $this->getConstraintViolation(Form::NOT_SYNCHRONIZED_ERROR, null);
+        $violation = $this->getConstraintViolation(Form::NOT_SYNCHRONIZED_ERROR);
         $form = $this->getForm('street');
 
         $this->validator->expects($this->once())
@@ -180,33 +160,11 @@ class ValidationListenerTest extends \PHPUnit_Framework_TestCase
         $this->listener->validateForm(new FormEvent($form, null));
     }
 
-    public function testValidatorInterfaceSinceSymfony25()
+    public function testValidatorInterface()
     {
-        // Mock of ValidatorInterface since apiVersion 2.5
-        $validator = $this->getMock('Symfony\Component\Validator\Validator\ValidatorInterface');
+        $validator = $this->getMockBuilder('Symfony\Component\Validator\Validator\ValidatorInterface')->getMock();
 
         $listener = new ValidationListener($validator, $this->violationMapper);
         $this->assertAttributeSame($validator, 'validator', $listener);
     }
-
-    /**
-     * @group legacy
-     */
-    public function testValidatorInterfaceUntilSymfony24()
-    {
-        // Mock of ValidatorInterface until apiVersion 2.4
-        $validator = $this->getMock('Symfony\Component\Validator\ValidatorInterface');
-
-        $listener = new ValidationListener($validator, $this->violationMapper);
-        $this->assertAttributeSame($validator, 'validator', $listener);
-    }
-
-    /**
-     * @group legacy
-     * @expectedException \InvalidArgumentException
-     */
-    public function testInvalidValidatorInterface()
-    {
-        new ValidationListener(null, $this->violationMapper);
-    }
 }
