@@ -37,8 +37,6 @@ class FullTransformer
     private $timezone;
 
     /**
-     * Constructor.
-     *
      * @param string $pattern  The pattern to be used to format and/or parse values
      * @param string $timezone The timezone to perform the date/time calculations
      */
@@ -67,7 +65,7 @@ class FullTransformer
             'k' => new Hour2401Transformer(),
             'm' => new MinuteTransformer(),
             's' => new SecondTransformer(),
-            'z' => new TimeZoneTransformer(),
+            'z' => new TimezoneTransformer(),
         );
     }
 
@@ -90,10 +88,8 @@ class FullTransformer
      */
     public function format(\DateTime $dateTime)
     {
-        $that = $this;
-
-        $formatted = preg_replace_callback($this->regExp, function ($matches) use ($that, $dateTime) {
-            return $that->formatReplace($matches[0], $dateTime);
+        $formatted = preg_replace_callback($this->regExp, function ($matches) use ($dateTime) {
+            return $this->formatReplace($matches[0], $dateTime);
         }, $this->pattern);
 
         return $formatted;
@@ -135,7 +131,7 @@ class FullTransformer
      * @param \DateTime $dateTime A configured DateTime object to use to perform the date calculation
      * @param string    $value    String to convert to a time value
      *
-     * @return int The corresponding Unix timestamp
+     * @return int|false The corresponding Unix timestamp
      *
      * @throws \InvalidArgumentException When the value can not be matched with pattern
      */
@@ -178,24 +174,22 @@ class FullTransformer
      */
     public function getReverseMatchingRegExp($pattern)
     {
-        $that = $this;
-
         $escapedPattern = preg_quote($pattern, '/');
 
         // ICU 4.8 recognizes slash ("/") in a value to be parsed as a dash ("-") and vice-versa
         // when parsing a date/time value
         $escapedPattern = preg_replace('/\\\[\-|\/]/', '[\/\-]', $escapedPattern);
 
-        $reverseMatchingRegExp = preg_replace_callback($this->regExp, function ($matches) use ($that) {
+        $reverseMatchingRegExp = preg_replace_callback($this->regExp, function ($matches) {
             $length = strlen($matches[0]);
             $transformerIndex = $matches[0][0];
 
             $dateChars = $matches[0];
-            if ($that->isQuoteMatch($dateChars)) {
-                return $that->replaceQuoteMatch($dateChars);
+            if ($this->isQuoteMatch($dateChars)) {
+                return $this->replaceQuoteMatch($dateChars);
             }
 
-            $transformers = $that->getTransformers();
+            $transformers = $this->getTransformers();
             if (isset($transformers[$transformerIndex])) {
                 $transformer = $transformers[$transformerIndex];
                 $captureName = str_repeat($transformerIndex, $length);
@@ -257,8 +251,6 @@ class FullTransformer
      * Normalize a preg_replace match array, removing the numeric keys and returning an associative array
      * with the value and pattern values for the matched Transformer.
      *
-     * @param array $data
-     *
      * @return array
      */
     protected function normalizeArray(array $data)
@@ -336,8 +328,6 @@ class FullTransformer
      * Add sensible default values for missing items in the extracted date/time options array. The values
      * are base in the beginning of the Unix era.
      *
-     * @param array $options
-     *
      * @return array
      */
     private function getDefaultValueForOptions(array $options)
