@@ -37,105 +37,107 @@ use Symfony\Component\Intl\Locale\Locale;
  *
  * @author Eriksen Costa <eriksen.costa@infranology.com.br>
  * @author Bernhard Schussek <bschussek@gmail.com>
+ *
+ * @internal
  */
 class NumberFormatter
 {
     /* Format style constants */
-    const PATTERN_DECIMAL   = 0;
-    const DECIMAL           = 1;
-    const CURRENCY          = 2;
-    const PERCENT           = 3;
-    const SCIENTIFIC        = 4;
-    const SPELLOUT          = 5;
-    const ORDINAL           = 6;
-    const DURATION          = 7;
+    const PATTERN_DECIMAL = 0;
+    const DECIMAL = 1;
+    const CURRENCY = 2;
+    const PERCENT = 3;
+    const SCIENTIFIC = 4;
+    const SPELLOUT = 5;
+    const ORDINAL = 6;
+    const DURATION = 7;
     const PATTERN_RULEBASED = 9;
-    const IGNORE            = 0;
-    const DEFAULT_STYLE     = 1;
+    const IGNORE = 0;
+    const DEFAULT_STYLE = 1;
 
     /* Format type constants */
-    const TYPE_DEFAULT  = 0;
-    const TYPE_INT32    = 1;
-    const TYPE_INT64    = 2;
-    const TYPE_DOUBLE   = 3;
+    const TYPE_DEFAULT = 0;
+    const TYPE_INT32 = 1;
+    const TYPE_INT64 = 2;
+    const TYPE_DOUBLE = 3;
     const TYPE_CURRENCY = 4;
 
     /* Numeric attribute constants */
-    const PARSE_INT_ONLY          = 0;
-    const GROUPING_USED           = 1;
-    const DECIMAL_ALWAYS_SHOWN    = 2;
-    const MAX_INTEGER_DIGITS      = 3;
-    const MIN_INTEGER_DIGITS      = 4;
-    const INTEGER_DIGITS          = 5;
-    const MAX_FRACTION_DIGITS     = 6;
-    const MIN_FRACTION_DIGITS     = 7;
-    const FRACTION_DIGITS         = 8;
-    const MULTIPLIER              = 9;
-    const GROUPING_SIZE           = 10;
-    const ROUNDING_MODE           = 11;
-    const ROUNDING_INCREMENT      = 12;
-    const FORMAT_WIDTH            = 13;
-    const PADDING_POSITION        = 14;
+    const PARSE_INT_ONLY = 0;
+    const GROUPING_USED = 1;
+    const DECIMAL_ALWAYS_SHOWN = 2;
+    const MAX_INTEGER_DIGITS = 3;
+    const MIN_INTEGER_DIGITS = 4;
+    const INTEGER_DIGITS = 5;
+    const MAX_FRACTION_DIGITS = 6;
+    const MIN_FRACTION_DIGITS = 7;
+    const FRACTION_DIGITS = 8;
+    const MULTIPLIER = 9;
+    const GROUPING_SIZE = 10;
+    const ROUNDING_MODE = 11;
+    const ROUNDING_INCREMENT = 12;
+    const FORMAT_WIDTH = 13;
+    const PADDING_POSITION = 14;
     const SECONDARY_GROUPING_SIZE = 15;
     const SIGNIFICANT_DIGITS_USED = 16;
-    const MIN_SIGNIFICANT_DIGITS  = 17;
-    const MAX_SIGNIFICANT_DIGITS  = 18;
-    const LENIENT_PARSE           = 19;
+    const MIN_SIGNIFICANT_DIGITS = 17;
+    const MAX_SIGNIFICANT_DIGITS = 18;
+    const LENIENT_PARSE = 19;
 
     /* Text attribute constants */
-    const POSITIVE_PREFIX   = 0;
-    const POSITIVE_SUFFIX   = 1;
-    const NEGATIVE_PREFIX   = 2;
-    const NEGATIVE_SUFFIX   = 3;
+    const POSITIVE_PREFIX = 0;
+    const POSITIVE_SUFFIX = 1;
+    const NEGATIVE_PREFIX = 2;
+    const NEGATIVE_SUFFIX = 3;
     const PADDING_CHARACTER = 4;
-    const CURRENCY_CODE     = 5;
-    const DEFAULT_RULESET   = 6;
-    const PUBLIC_RULESETS   = 7;
+    const CURRENCY_CODE = 5;
+    const DEFAULT_RULESET = 6;
+    const PUBLIC_RULESETS = 7;
 
     /* Format symbol constants */
-    const DECIMAL_SEPARATOR_SYMBOL           = 0;
-    const GROUPING_SEPARATOR_SYMBOL          = 1;
-    const PATTERN_SEPARATOR_SYMBOL           = 2;
-    const PERCENT_SYMBOL                     = 3;
-    const ZERO_DIGIT_SYMBOL                  = 4;
-    const DIGIT_SYMBOL                       = 5;
-    const MINUS_SIGN_SYMBOL                  = 6;
-    const PLUS_SIGN_SYMBOL                   = 7;
-    const CURRENCY_SYMBOL                    = 8;
-    const INTL_CURRENCY_SYMBOL               = 9;
-    const MONETARY_SEPARATOR_SYMBOL          = 10;
-    const EXPONENTIAL_SYMBOL                 = 11;
-    const PERMILL_SYMBOL                     = 12;
-    const PAD_ESCAPE_SYMBOL                  = 13;
-    const INFINITY_SYMBOL                    = 14;
-    const NAN_SYMBOL                         = 15;
-    const SIGNIFICANT_DIGIT_SYMBOL           = 16;
+    const DECIMAL_SEPARATOR_SYMBOL = 0;
+    const GROUPING_SEPARATOR_SYMBOL = 1;
+    const PATTERN_SEPARATOR_SYMBOL = 2;
+    const PERCENT_SYMBOL = 3;
+    const ZERO_DIGIT_SYMBOL = 4;
+    const DIGIT_SYMBOL = 5;
+    const MINUS_SIGN_SYMBOL = 6;
+    const PLUS_SIGN_SYMBOL = 7;
+    const CURRENCY_SYMBOL = 8;
+    const INTL_CURRENCY_SYMBOL = 9;
+    const MONETARY_SEPARATOR_SYMBOL = 10;
+    const EXPONENTIAL_SYMBOL = 11;
+    const PERMILL_SYMBOL = 12;
+    const PAD_ESCAPE_SYMBOL = 13;
+    const INFINITY_SYMBOL = 14;
+    const NAN_SYMBOL = 15;
+    const SIGNIFICANT_DIGIT_SYMBOL = 16;
     const MONETARY_GROUPING_SEPARATOR_SYMBOL = 17;
 
     /* Rounding mode values used by NumberFormatter::setAttribute() with NumberFormatter::ROUNDING_MODE attribute */
-    const ROUND_CEILING  = 0;
-    const ROUND_FLOOR    = 1;
-    const ROUND_DOWN     = 2;
-    const ROUND_UP       = 3;
+    const ROUND_CEILING = 0;
+    const ROUND_FLOOR = 1;
+    const ROUND_DOWN = 2;
+    const ROUND_UP = 3;
     const ROUND_HALFEVEN = 4;
     const ROUND_HALFDOWN = 5;
-    const ROUND_HALFUP   = 6;
+    const ROUND_HALFUP = 6;
 
     /* Pad position values used by NumberFormatter::setAttribute() with NumberFormatter::PADDING_POSITION attribute */
     const PAD_BEFORE_PREFIX = 0;
-    const PAD_AFTER_PREFIX  = 1;
+    const PAD_AFTER_PREFIX = 1;
     const PAD_BEFORE_SUFFIX = 2;
-    const PAD_AFTER_SUFFIX  = 3;
+    const PAD_AFTER_SUFFIX = 3;
 
     /**
-     * The error code from the last operation
+     * The error code from the last operation.
      *
      * @var int
      */
     protected $errorCode = IntlGlobals::U_ZERO_ERROR;
 
     /**
-     * The error message from the last operation
+     * The error message from the last operation.
      *
      * @var string
      */
@@ -147,55 +149,49 @@ class NumberFormatter
     private $style;
 
     /**
-     * Default values for the en locale
-     *
-     * @var array
+     * Default values for the en locale.
      */
     private $attributes = array(
         self::FRACTION_DIGITS => 0,
-        self::GROUPING_USED   => 1,
-        self::ROUNDING_MODE   => self::ROUND_HALFEVEN,
+        self::GROUPING_USED => 1,
+        self::ROUNDING_MODE => self::ROUND_HALFEVEN,
     );
 
     /**
-     * Holds the initialized attributes code
-     *
-     * @var array
+     * Holds the initialized attributes code.
      */
     private $initializedAttributes = array();
 
     /**
-     * The supported styles to the constructor $styles argument
-     *
-     * @var array
+     * The supported styles to the constructor $styles argument.
      */
     private static $supportedStyles = array(
         'CURRENCY' => self::CURRENCY,
-        'DECIMAL'  => self::DECIMAL,
+        'DECIMAL' => self::DECIMAL,
     );
 
     /**
-     * Supported attributes to the setAttribute() $attr argument
-     *
-     * @var array
+     * Supported attributes to the setAttribute() $attr argument.
      */
     private static $supportedAttributes = array(
         'FRACTION_DIGITS' => self::FRACTION_DIGITS,
-        'GROUPING_USED'   => self::GROUPING_USED,
-        'ROUNDING_MODE'   => self::ROUNDING_MODE,
+        'GROUPING_USED' => self::GROUPING_USED,
+        'ROUNDING_MODE' => self::ROUNDING_MODE,
     );
 
     /**
      * The available rounding modes for setAttribute() usage with
      * NumberFormatter::ROUNDING_MODE. NumberFormatter::ROUND_DOWN
-     * and NumberFormatter::ROUND_UP does not have a PHP only equivalent
-     *
-     * @var array
+     * and NumberFormatter::ROUND_UP does not have a PHP only equivalent.
      */
     private static $roundingModes = array(
         'ROUND_HALFEVEN' => self::ROUND_HALFEVEN,
         'ROUND_HALFDOWN' => self::ROUND_HALFDOWN,
-        'ROUND_HALFUP'   => self::ROUND_HALFUP,
+        'ROUND_HALFUP' => self::ROUND_HALFUP,
+        'ROUND_CEILING' => self::ROUND_CEILING,
+        'ROUND_FLOOR' => self::ROUND_FLOOR,
+        'ROUND_DOWN' => self::ROUND_DOWN,
+        'ROUND_UP' => self::ROUND_UP,
     );
 
     /**
@@ -203,34 +199,36 @@ class NumberFormatter
      * modes in PHP's round() function.
      *
      * @see http://www.php.net/manual/en/function.round.php
-     *
-     * @var array
      */
     private static $phpRoundingMap = array(
         self::ROUND_HALFDOWN => \PHP_ROUND_HALF_DOWN,
         self::ROUND_HALFEVEN => \PHP_ROUND_HALF_EVEN,
-        self::ROUND_HALFUP   => \PHP_ROUND_HALF_UP,
+        self::ROUND_HALFUP => \PHP_ROUND_HALF_UP,
     );
 
     /**
-     * The maximum values of the integer type in 32 bit platforms.
-     *
-     * @var array
+     * The list of supported rounding modes which aren't available modes in
+     * PHP's round() function, but there's an equivalent. Keys are rounding
+     * modes, values does not matter.
      */
-    private static $int32Range = array(
-        'positive' => 2147483647,
-        'negative' => -2147483648,
+    private static $customRoundingList = array(
+        self::ROUND_CEILING => true,
+        self::ROUND_FLOOR => true,
+        self::ROUND_DOWN => true,
+        self::ROUND_UP => true,
     );
 
     /**
-     * The maximum values of the integer type in 64 bit platforms.
-     *
-     * @var array
+     * The maximum value of the integer type in 32 bit platforms.
      */
-    private static $int64Range = array(
-        'positive' => 9223372036854775807,
-        'negative' => -9223372036854775808,
-    );
+    private static $int32Max = 2147483647;
+
+    /**
+     * The maximum value of the integer type in 64 bit platforms.
+     *
+     * @var int|float
+     */
+    private static $int64Max = 9223372036854775807;
 
     private static $enSymbols = array(
         self::DECIMAL => array('.', ',', ';', '%', '0', '#', '-', '+', '¤', '¤¤', '.', 'E', '‰', '*', '∞', 'NaN', '@', ','),
@@ -238,14 +236,12 @@ class NumberFormatter
     );
 
     private static $enTextAttributes = array(
-        self::DECIMAL => array('', '', '-', '', '*', '', ''),
-        self::CURRENCY => array('¤', '', '(¤', ')', '*', ''),
+        self::DECIMAL => array('', '', '-', '', ' ', '', ''),
+        self::CURRENCY => array('¤', '', '-¤', '', ' ', ''),
     );
 
     /**
-     * Constructor.
-     *
-     * @param string $locale  The locale code. The only currently supported locale is "en".
+     * @param string $locale  The locale code. The only currently supported locale is "en" (or null using the default locale, i.e. "en")
      * @param int    $style   Style of the formatting, one of the format style constants.
      *                        The only supported styles are NumberFormatter::DECIMAL
      *                        and NumberFormatter::CURRENCY.
@@ -257,13 +253,13 @@ class NumberFormatter
      * @see http://www.icu-project.org/apiref/icu4c/classDecimalFormat.html#_details
      * @see http://www.icu-project.org/apiref/icu4c/classRuleBasedNumberFormat.html#_details
      *
-     * @throws MethodArgumentValueNotImplementedException  When $locale different than "en" is passed
-     * @throws MethodArgumentValueNotImplementedException  When the $style is not supported
-     * @throws MethodArgumentNotImplementedException       When the pattern value is different than null
+     * @throws MethodArgumentValueNotImplementedException When $locale different than "en" or null is passed
+     * @throws MethodArgumentValueNotImplementedException When the $style is not supported
+     * @throws MethodArgumentNotImplementedException      When the pattern value is different than null
      */
     public function __construct($locale = 'en', $style = null, $pattern = null)
     {
-        if ('en' != $locale) {
+        if ('en' !== $locale && null !== $locale) {
             throw new MethodArgumentValueNotImplementedException(__METHOD__, 'locale', $locale, 'Only the locale "en" is supported');
         }
 
@@ -276,13 +272,13 @@ class NumberFormatter
             throw new MethodArgumentNotImplementedException(__METHOD__, 'pattern');
         }
 
-        $this->style  = $style;
+        $this->style = $style;
     }
 
     /**
      * Static constructor.
      *
-     * @param string $locale  The locale code. The only supported locale is "en".
+     * @param string $locale  The locale code. The only supported locale is "en" (or null using the default locale, i.e. "en")
      * @param int    $style   Style of the formatting, one of the format style constants.
      *                        The only currently supported styles are NumberFormatter::DECIMAL
      *                        and NumberFormatter::CURRENCY.
@@ -290,15 +286,15 @@ class NumberFormatter
      *                        NumberFormat::PATTERN_RULEBASED. It must conform to  the syntax
      *                        described in the ICU DecimalFormat or ICU RuleBasedNumberFormat documentation
      *
-     * @return NumberFormatter
+     * @return self
      *
      * @see http://www.php.net/manual/en/numberformatter.create.php
      * @see http://www.icu-project.org/apiref/icu4c/classDecimalFormat.html#_details
      * @see http://www.icu-project.org/apiref/icu4c/classRuleBasedNumberFormat.html#_details
      *
-     * @throws MethodArgumentValueNotImplementedException  When $locale different than "en" is passed
-     * @throws MethodArgumentValueNotImplementedException  When the $style is not supported
-     * @throws MethodArgumentNotImplementedException       When the pattern value is different than null
+     * @throws MethodArgumentValueNotImplementedException When $locale different than "en" or null is passed
+     * @throws MethodArgumentValueNotImplementedException When the $style is not supported
+     * @throws MethodArgumentNotImplementedException      When the pattern value is different than null
      */
     public static function create($locale = 'en', $style = null, $pattern = null)
     {
@@ -306,7 +302,7 @@ class NumberFormatter
     }
 
     /**
-     * Format a currency value
+     * Format a currency value.
      *
      * @param float  $value    The numeric currency value
      * @param string $currency The 3-letter ISO 4217 currency code indicating the currency to use
@@ -314,11 +310,11 @@ class NumberFormatter
      * @return string The formatted currency value
      *
      * @see http://www.php.net/manual/en/numberformatter.formatcurrency.php
-     * @see http://www.iso.org/iso/support/faqs/faqs_widely_used_standards/widely_used_standards_other/currency_codes/currency_codes_list-1.htm
+     * @see https://en.wikipedia.org/wiki/ISO_4217#Active_codes
      */
     public function formatCurrency($value, $currency)
     {
-        if ($this->style == self::DECIMAL) {
+        if (self::DECIMAL == $this->style) {
             return $this->format($value);
         }
 
@@ -337,17 +333,17 @@ class NumberFormatter
 
         $ret = $symbol.$value;
 
-        return $negative ? '('.$ret.')' : $ret;
+        return $negative ? '-'.$ret : $ret;
     }
 
     /**
-     * Format a number
+     * Format a number.
      *
-     * @param number $value The value to format
-     * @param int    $type  Type of the formatting, one of the format type constants.
-     *                      Only type NumberFormatter::TYPE_DEFAULT is currently supported.
+     * @param int|float $value The value to format
+     * @param int       $type  Type of the formatting, one of the format type constants.
+     *                         Only type NumberFormatter::TYPE_DEFAULT is currently supported.
      *
-     * @return bool|string    The formatted value or false on error
+     * @return bool|string The formatted value or false on error
      *
      * @see http://www.php.net/manual/en/numberformatter.format.php
      *
@@ -357,13 +353,13 @@ class NumberFormatter
     public function format($value, $type = self::TYPE_DEFAULT)
     {
         // The original NumberFormatter does not support this format type
-        if ($type == self::TYPE_CURRENCY) {
+        if (self::TYPE_CURRENCY == $type) {
             trigger_error(__METHOD__.'(): Unsupported format type '.$type, \E_USER_WARNING);
 
             return false;
         }
 
-        if ($this->style == self::CURRENCY) {
+        if (self::CURRENCY == $this->style) {
             throw new NotImplementedException(sprintf(
                 '%s() method does not support the formatting of currencies (instance with CURRENCY style). %s',
                 __METHOD__, NotImplementedException::INTL_INSTALL_MESSAGE
@@ -371,7 +367,7 @@ class NumberFormatter
         }
 
         // Only the default type is supported.
-        if ($type != self::TYPE_DEFAULT) {
+        if (self::TYPE_DEFAULT != $type) {
             throw new MethodArgumentValueNotImplementedException(__METHOD__, 'type', $type, 'Only TYPE_DEFAULT is supported');
         }
 
@@ -387,11 +383,11 @@ class NumberFormatter
     }
 
     /**
-     * Returns an attribute value
+     * Returns an attribute value.
      *
      * @param int $attr An attribute specifier, one of the numeric attribute constants
      *
-     * @return bool|int    The attribute value on success or false on error
+     * @return bool|int The attribute value on success or false on error
      *
      * @see http://www.php.net/manual/en/numberformatter.getattribute.php
      */
@@ -401,7 +397,7 @@ class NumberFormatter
     }
 
     /**
-     * Returns formatter's last error code. Always returns the U_ZERO_ERROR class constant value
+     * Returns formatter's last error code. Always returns the U_ZERO_ERROR class constant value.
      *
      * @return int The error code from last formatter call
      *
@@ -413,7 +409,7 @@ class NumberFormatter
     }
 
     /**
-     * Returns formatter's last error message. Always returns the U_ZERO_ERROR_MESSAGE class constant value
+     * Returns formatter's last error message. Always returns the U_ZERO_ERROR_MESSAGE class constant value.
      *
      * @return string The error message from last formatter call
      *
@@ -425,7 +421,7 @@ class NumberFormatter
     }
 
     /**
-     * Returns the formatter's locale
+     * Returns the formatter's locale.
      *
      * The parameter $type is currently ignored.
      *
@@ -442,9 +438,9 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Returns the formatter's pattern
+     * Not supported. Returns the formatter's pattern.
      *
-     * @return bool|string        The pattern string used by the formatter or false on error
+     * @return bool|string The pattern string used by the formatter or false on error
      *
      * @see http://www.php.net/manual/en/numberformatter.getpattern.php
      *
@@ -456,11 +452,11 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Returns a formatter symbol value
+     * Not supported. Returns a formatter symbol value.
      *
      * @param int $attr A symbol specifier, one of the format symbol constants
      *
-     * @return bool|string           The symbol value or false on error
+     * @return bool|string The symbol value or false on error
      *
      * @see http://www.php.net/manual/en/numberformatter.getsymbol.php
      */
@@ -470,11 +466,11 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Returns a formatter text attribute value
+     * Not supported. Returns a formatter text attribute value.
      *
      * @param int $attr An attribute specifier, one of the text attribute constants
      *
-     * @return bool|string           The attribute value or false on error
+     * @return bool|string The attribute value or false on error
      *
      * @see http://www.php.net/manual/en/numberformatter.gettextattribute.php
      */
@@ -484,13 +480,13 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Parse a currency number
+     * Not supported. Parse a currency number.
      *
      * @param string $value    The value to parse
      * @param string $currency Parameter to receive the currency name (reference)
      * @param int    $position Offset to begin the parsing on return this value will hold the offset at which the parsing ended
      *
-     * @return bool|string              The parsed numeric value of false on error
+     * @return bool|string The parsed numeric value of false on error
      *
      * @see http://www.php.net/manual/en/numberformatter.parsecurrency.php
      *
@@ -502,40 +498,48 @@ class NumberFormatter
     }
 
     /**
-     * Parse a number
+     * Parse a number.
      *
      * @param string $value    The value to parse
-     * @param int    $type     Type of the formatting, one of the format type constants. NumberFormatter::TYPE_DOUBLE by default
+     * @param int    $type     Type of the formatting, one of the format type constants. NumberFormatter::TYPE_DOUBLE by default.
      * @param int    $position Offset to begin the parsing on return this value will hold the offset at which the parsing ended
      *
-     * @return bool|string                                  The parsed value of false on error
+     * @return int|float|false The parsed value of false on error
      *
-     * @see    http://www.php.net/manual/en/numberformatter.parse.php
+     * @see http://www.php.net/manual/en/numberformatter.parse.php
      */
     public function parse($value, $type = self::TYPE_DOUBLE, &$position = 0)
     {
-        if ($type == self::TYPE_DEFAULT || $type == self::TYPE_CURRENCY) {
+        if (self::TYPE_DEFAULT == $type || self::TYPE_CURRENCY == $type) {
             trigger_error(__METHOD__.'(): Unsupported format type '.$type, \E_USER_WARNING);
 
             return false;
         }
 
-        preg_match('/^([^0-9\-\.]{0,})(.*)/', $value, $matches);
+        $groupSep = $this->getAttribute(self::GROUPING_USED) ? ',' : '';
 
         // Any string before the numeric value causes error in the parsing
-        if (isset($matches[1]) && !empty($matches[1])) {
+        if (preg_match("/^-?(?:\.\d++|([\d{$groupSep}]++)(?:\.\d++)?)/", $value, $matches)) {
+            $value = $matches[0];
+            $position = strlen($value);
+            if ($error = $groupSep && isset($matches[1]) && !preg_match('/^\d{1,3}+(?:(?:,\d{3})++|\d*+)$/', $matches[1])) {
+                $position -= strlen(preg_replace('/^\d{1,3}+(?:(?:,\d++)++|\d*+)/', '', $matches[1]));
+            }
+        } else {
+            $error = 1;
+            $position = 0;
+        }
+
+        if ($error) {
             IntlGlobals::setError(IntlGlobals::U_PARSE_ERROR, 'Number parsing failed');
             $this->errorCode = IntlGlobals::getErrorCode();
             $this->errorMessage = IntlGlobals::getErrorMessage();
-            $position = 0;
 
             return false;
         }
 
-        preg_match('/^[0-9\-\.\,]*/', $value, $matches);
-        $value = preg_replace('/[^0-9\.\-]/', '', $matches[0]);
+        $value = str_replace(',', '', $value);
         $value = $this->convertValueDataType($value, $type);
-        $position = strlen($matches[0]);
 
         // behave like the intl extension
         $this->resetError();
@@ -544,21 +548,19 @@ class NumberFormatter
     }
 
     /**
-     * Set an attribute
+     * Set an attribute.
      *
      * @param int $attr  An attribute specifier, one of the numeric attribute constants.
      *                   The only currently supported attributes are NumberFormatter::FRACTION_DIGITS,
      *                   NumberFormatter::GROUPING_USED and NumberFormatter::ROUNDING_MODE.
-     * @param int $value The attribute value. The only currently supported rounding modes are
-     *                   NumberFormatter::ROUND_HALFEVEN, NumberFormatter::ROUND_HALFDOWN and
-     *                   NumberFormatter::ROUND_HALFUP.
+     * @param int $value The attribute value
      *
-     * @return bool    true on success or false on failure
+     * @return bool true on success or false on failure
      *
      * @see http://www.php.net/manual/en/numberformatter.setattribute.php
      *
-     * @throws MethodArgumentValueNotImplementedException  When the $attr is not supported
-     * @throws MethodArgumentValueNotImplementedException  When the $value is not supported
+     * @throws MethodArgumentValueNotImplementedException When the $attr is not supported
+     * @throws MethodArgumentValueNotImplementedException When the $value is not supported
      */
     public function setAttribute($attr, $value)
     {
@@ -595,11 +597,11 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Set the formatter's pattern
+     * Not supported. Set the formatter's pattern.
      *
      * @param string $pattern A pattern string in conformance with the ICU DecimalFormat documentation
      *
-     * @return bool    true on success or false on failure
+     * @return bool true on success or false on failure
      *
      * @see http://www.php.net/manual/en/numberformatter.setpattern.php
      * @see http://www.icu-project.org/apiref/icu4c/classDecimalFormat.html#_details
@@ -612,12 +614,12 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Set the formatter's symbol
+     * Not supported. Set the formatter's symbol.
      *
      * @param int    $attr  A symbol specifier, one of the format symbol constants
      * @param string $value The value for the symbol
      *
-     * @return bool    true on success or false on failure
+     * @return bool true on success or false on failure
      *
      * @see http://www.php.net/manual/en/numberformatter.setsymbol.php
      *
@@ -629,12 +631,12 @@ class NumberFormatter
     }
 
     /**
-     * Not supported. Set a text attribute
+     * Not supported. Set a text attribute.
      *
      * @param int $attr  An attribute specifier, one of the text attribute constants
      * @param int $value The attribute value
      *
-     * @return bool    true on success or false on failure
+     * @return bool true on success or false on failure
      *
      * @see http://www.php.net/manual/en/numberformatter.settextattribute.php
      *
@@ -646,7 +648,7 @@ class NumberFormatter
     }
 
     /**
-     * Set the error to the default U_ZERO_ERROR
+     * Set the error to the default U_ZERO_ERROR.
      */
     protected function resetError()
     {
@@ -656,7 +658,7 @@ class NumberFormatter
     }
 
     /**
-     * Rounds a currency value, applying increment rounding if applicable
+     * Rounds a currency value, applying increment rounding if applicable.
      *
      * When a currency have a rounding increment, an extra round is made after the first one. The rounding factor is
      * determined in the ICU data and is explained as of:
@@ -668,7 +670,7 @@ class NumberFormatter
      * @param float  $value    The numeric currency value
      * @param string $currency The 3-letter ISO 4217 currency code indicating the currency to use
      *
-     * @return string The rounded numeric currency value
+     * @return float The rounded numeric currency value
      *
      * @see http://en.wikipedia.org/wiki/Swedish_rounding
      * @see http://www.docjar.com/html/api/com/ibm/icu/util/Currency.java.html#1007
@@ -693,17 +695,39 @@ class NumberFormatter
     /**
      * Rounds a value.
      *
-     * @param int|float     $value     The value to round
-     * @param int           $precision The number of decimal digits to round to
+     * @param int|float $value     The value to round
+     * @param int       $precision The number of decimal digits to round to
      *
-     * @return int|float     The rounded value
+     * @return int|float The rounded value
      */
     private function round($value, $precision)
     {
-        $precision = $this->getUnitializedPrecision($value, $precision);
+        $precision = $this->getUninitializedPrecision($value, $precision);
 
-        $roundingMode = self::$phpRoundingMap[$this->getAttribute(self::ROUNDING_MODE)];
-        $value = round($value, $precision, $roundingMode);
+        $roundingModeAttribute = $this->getAttribute(self::ROUNDING_MODE);
+        if (isset(self::$phpRoundingMap[$roundingModeAttribute])) {
+            $value = round($value, $precision, self::$phpRoundingMap[$roundingModeAttribute]);
+        } elseif (isset(self::$customRoundingList[$roundingModeAttribute])) {
+            $roundingCoef = pow(10, $precision);
+            $value *= $roundingCoef;
+
+            switch ($roundingModeAttribute) {
+                case self::ROUND_CEILING:
+                    $value = ceil($value);
+                    break;
+                case self::ROUND_FLOOR:
+                    $value = floor($value);
+                    break;
+                case self::ROUND_UP:
+                    $value = $value > 0 ? ceil($value) : floor($value);
+                    break;
+                case self::ROUND_DOWN:
+                    $value = $value > 0 ? floor($value) : ceil($value);
+                    break;
+            }
+
+            $value /= $roundingCoef;
+        }
 
         return $value;
     }
@@ -711,29 +735,29 @@ class NumberFormatter
     /**
      * Formats a number.
      *
-     * @param int|float     $value     The numeric value to format
-     * @param int           $precision The number of decimal digits to use
+     * @param int|float $value     The numeric value to format
+     * @param int       $precision The number of decimal digits to use
      *
      * @return string The formatted number
      */
     private function formatNumber($value, $precision)
     {
-        $precision = $this->getUnitializedPrecision($value, $precision);
+        $precision = $this->getUninitializedPrecision($value, $precision);
 
         return number_format($value, $precision, '.', $this->getAttribute(self::GROUPING_USED) ? ',' : '');
     }
 
     /**
-     * Returns the precision value if the DECIMAL style is being used and the FRACTION_DIGITS attribute is unitialized.
+     * Returns the precision value if the DECIMAL style is being used and the FRACTION_DIGITS attribute is uninitialized.
      *
-     * @param int|float     $value     The value to get the precision from if the FRACTION_DIGITS attribute is unitialized
-     * @param int           $precision The precision value to returns if the FRACTION_DIGITS attribute is initialized
+     * @param int|float $value     The value to get the precision from if the FRACTION_DIGITS attribute is uninitialized
+     * @param int       $precision The precision value to returns if the FRACTION_DIGITS attribute is initialized
      *
      * @return int The precision value
      */
-    private function getUnitializedPrecision($value, $precision)
+    private function getUninitializedPrecision($value, $precision)
     {
-        if ($this->style == self::CURRENCY) {
+        if (self::CURRENCY == $this->style) {
             return $precision;
         }
 
@@ -752,7 +776,7 @@ class NumberFormatter
      *
      * @param string $attr The attribute name
      *
-     * @return bool    true if the value was set by client, false otherwise
+     * @return bool true if the value was set by client, false otherwise
      */
     private function isInitializedAttribute($attr)
     {
@@ -765,15 +789,15 @@ class NumberFormatter
      * @param mixed $value The value to be converted
      * @param int   $type  The type to convert. Can be TYPE_DOUBLE (float) or TYPE_INT32 (int)
      *
-     * @return int|float     The converted value
+     * @return int|float|false The converted value
      */
     private function convertValueDataType($value, $type)
     {
-        if ($type == self::TYPE_DOUBLE) {
+        if (self::TYPE_DOUBLE == $type) {
             $value = (float) $value;
-        } elseif ($type == self::TYPE_INT32) {
+        } elseif (self::TYPE_INT32 == $type) {
             $value = $this->getInt32Value($value);
-        } elseif ($type == self::TYPE_INT64) {
+        } elseif (self::TYPE_INT64 == $type) {
             $value = $this->getInt64Value($value);
         }
 
@@ -785,11 +809,11 @@ class NumberFormatter
      *
      * @param mixed $value The value to be converted
      *
-     * @return int The converted value
+     * @return int|false The converted value
      */
     private function getInt32Value($value)
     {
-        if ($value > self::$int32Range['positive'] || $value < self::$int32Range['negative']) {
+        if ($value > self::$int32Max || $value < -self::$int32Max - 1) {
             return false;
         }
 
@@ -801,46 +825,18 @@ class NumberFormatter
      *
      * @param mixed $value The value to be converted
      *
-     * @return int|float       The converted value
-     *
-     * @see https://bugs.php.net/bug.php?id=59597 Bug #59597
+     * @return int|float|false The converted value
      */
     private function getInt64Value($value)
     {
-        if ($value > self::$int64Range['positive'] || $value < self::$int64Range['negative']) {
+        if ($value > self::$int64Max || $value < -self::$int64Max - 1) {
             return false;
         }
 
-        if (PHP_INT_SIZE !== 8 && ($value > self::$int32Range['positive'] || $value <= self::$int32Range['negative'])) {
-            // Bug #59597 was fixed on PHP 5.3.14 and 5.4.4
-            // The negative PHP_INT_MAX was being converted to float
-            if (
-                $value == self::$int32Range['negative'] &&
-                (
-                    (version_compare(PHP_VERSION, '5.4.0', '<') && version_compare(PHP_VERSION, '5.3.14', '>=')) ||
-                    version_compare(PHP_VERSION, '5.4.4', '>=')
-                )
-            ) {
-                return (int) $value;
-            }
-
+        if (PHP_INT_SIZE !== 8 && ($value > self::$int32Max || $value < -self::$int32Max - 1)) {
             return (float) $value;
         }
 
-        if (PHP_INT_SIZE === 8) {
-            // Bug #59597 was fixed on PHP 5.3.14 and 5.4.4
-            // A 32 bit integer was being generated instead of a 64 bit integer
-            if (
-                  ($value > self::$int32Range['positive'] || $value < self::$int32Range['negative']) &&
-                  (
-                      (version_compare(PHP_VERSION, '5.3.14', '<')) ||
-                      (version_compare(PHP_VERSION, '5.4.0', '>=') && version_compare(PHP_VERSION, '5.4.4', '<'))
-                  )
-            ) {
-                $value = (-2147483648 - ($value % -2147483648)) * ($value / abs($value));
-            }
-        }
-
         return (int) $value;
     }
 
@@ -849,7 +845,7 @@ class NumberFormatter
      *
      * @param int $value The rounding mode value to check
      *
-     * @return bool    true if the rounding mode is invalid, false otherwise
+     * @return bool true if the rounding mode is invalid, false otherwise
      */
     private function isInvalidRoundingMode($value)
     {
