@@ -52,6 +52,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'aro',
         'arp',
         'arq',
+        'ars',
         'arw',
         'ary',
         'arz',
@@ -64,7 +65,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'awa',
         'ay',
         'az',
-        'azb',
+        'az_Arab',
         'ba',
         'bal',
         'ban',
@@ -81,6 +82,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'bfd',
         'bfq',
         'bg',
+        'bgn',
         'bho',
         'bi',
         'bik',
@@ -109,6 +111,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'car',
         'cay',
         'cch',
+        'ccp',
         'ce',
         'ceb',
         'cgg',
@@ -128,6 +131,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'cps',
         'cr',
         'crh',
+        'crs',
         'cs',
         'csb',
         'cu',
@@ -180,6 +184,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'ewo',
         'ext',
         'fa',
+        'fa_AF',
         'fan',
         'fat',
         'ff',
@@ -339,7 +344,9 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'ln',
         'lo',
         'lol',
+        'lou',
         'loz',
+        'lrc',
         'lt',
         'ltg',
         'lu',
@@ -401,6 +408,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'nb',
         'nd',
         'nds',
+        'nds_NL',
         'ne',
         'new',
         'ng',
@@ -441,6 +449,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'pap',
         'pau',
         'pcd',
+        'pcm',
         'pdc',
         'pdt',
         'peo',
@@ -494,6 +503,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'sco',
         'sd',
         'sdc',
+        'sdh',
         'se',
         'see',
         'seh',
@@ -536,8 +546,8 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'sux',
         'sv',
         'sw',
+        'sw_CD',
         'swb',
-        'swc',
         'syc',
         'syr',
         'szl',
@@ -604,6 +614,7 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'wal',
         'war',
         'was',
+        'wbp',
         'wo',
         'wuu',
         'xal',
@@ -633,173 +644,173 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'zza',
     );
 
-    protected static $alpha2ToAlpha3 = array (
+    protected static $alpha2ToAlpha3 = array(
         'aa' => 'aar',
         'ab' => 'abk',
-        'ae' => 'ave',
         'af' => 'afr',
         'ak' => 'aka',
+        'sq' => 'sqi',
         'am' => 'amh',
-        'an' => 'arg',
         'ar' => 'ara',
+        'an' => 'arg',
+        'hy' => 'hye',
         'as' => 'asm',
         'av' => 'ava',
+        'ae' => 'ave',
         'ay' => 'aym',
         'az' => 'aze',
         'ba' => 'bak',
-        'be' => 'bel',
-        'bg' => 'bul',
-        'bh' => 'bih',
-        'bi' => 'bis',
         'bm' => 'bam',
+        'eu' => 'eus',
+        'be' => 'bel',
         'bn' => 'ben',
+        'bi' => 'bis',
         'bo' => 'bod',
-        'br' => 'bre',
         'bs' => 'bos',
+        'br' => 'bre',
+        'bg' => 'bul',
+        'my' => 'mya',
         'ca' => 'cat',
-        'ce' => 'che',
-        'ch' => 'cha',
-        'co' => 'cos',
-        'cr' => 'cre',
         'cs' => 'ces',
+        'ch' => 'cha',
+        'ce' => 'che',
+        'zh' => 'zho',
         'cu' => 'chu',
         'cv' => 'chv',
+        'kw' => 'cor',
+        'co' => 'cos',
+        'cr' => 'cre',
         'cy' => 'cym',
         'da' => 'dan',
         'de' => 'deu',
         'dv' => 'div',
+        'nl' => 'nld',
         'dz' => 'dzo',
-        'ee' => 'ewe',
+        'et' => 'est',
         'el' => 'ell',
         'en' => 'eng',
         'eo' => 'epo',
-        'es' => 'spa',
-        'et' => 'est',
-        'eu' => 'eus',
-        'fa' => 'fas',
-        'ff' => 'ful',
-        'fi' => 'fin',
-        'fj' => 'fij',
+        'ik' => 'ipk',
+        'ee' => 'ewe',
         'fo' => 'fao',
+        'fa' => 'fas',
+        'fj' => 'fij',
+        'fi' => 'fin',
         'fr' => 'fra',
         'fy' => 'fry',
-        'ga' => 'gle',
+        'ff' => 'ful',
+        'om' => 'orm',
+        'ka' => 'kat',
         'gd' => 'gla',
+        'ga' => 'gle',
         'gl' => 'glg',
+        'gv' => 'glv',
         'gn' => 'grn',
         'gu' => 'guj',
-        'gv' => 'glv',
+        'ht' => 'hat',
         'ha' => 'hau',
         'he' => 'heb',
+        'hz' => 'her',
         'hi' => 'hin',
         'ho' => 'hmo',
         'hr' => 'hrv',
-        'ht' => 'hat',
         'hu' => 'hun',
-        'hy' => 'hye',
-        'hz' => 'her',
+        'ig' => 'ibo',
+        'is' => 'isl',
+        'io' => 'ido',
+        'ii' => 'iii',
+        'iu' => 'iku',
+        'ie' => 'ile',
         'ia' => 'ina',
         'id' => 'ind',
-        'ie' => 'ile',
-        'ig' => 'ibo',
-        'ii' => 'iii',
-        'ik' => 'ipk',
-        'io' => 'ido',
-        'is' => 'isl',
         'it' => 'ita',
-        'iu' => 'iku',
-        'ja' => 'jpn',
         'jv' => 'jav',
-        'ka' => 'kat',
-        'kg' => 'kon',
-        'ki' => 'kik',
-        'kj' => 'kua',
-        'kk' => 'kaz',
+        'ja' => 'jpn',
         'kl' => 'kal',
-        'km' => 'khm',
         'kn' => 'kan',
-        'ko' => 'kor',
-        'kr' => 'kau',
         'ks' => 'kas',
-        'ku' => 'kur',
-        'kv' => 'kom',
-        'kw' => 'cor',
+        'kr' => 'kau',
+        'kk' => 'kaz',
+        'mn' => 'mon',
+        'km' => 'khm',
+        'ki' => 'kik',
+        'rw' => 'kin',
         'ky' => 'kir',
+        'ku' => 'kur',
+        'kg' => 'kon',
+        'kv' => 'kom',
+        'ko' => 'kor',
+        'kj' => 'kua',
+        'lo' => 'lao',
         'la' => 'lat',
-        'lb' => 'ltz',
-        'lg' => 'lug',
+        'lv' => 'lav',
         'li' => 'lim',
         'ln' => 'lin',
-        'lo' => 'lao',
         'lt' => 'lit',
+        'lb' => 'ltz',
         'lu' => 'lub',
-        'lv' => 'lav',
-        'mg' => 'mlg',
-        'mh' => 'mah',
-        'mi' => 'mri',
+        'lg' => 'lug',
         'mk' => 'mkd',
+        'mh' => 'mah',
         'ml' => 'mal',
-        'mn' => 'mon',
+        'mi' => 'mri',
         'mr' => 'mar',
         'ms' => 'msa',
+        'mg' => 'mlg',
         'mt' => 'mlt',
-        'my' => 'mya',
         'na' => 'nau',
-        'nb' => 'nob',
-        'nd' => 'nde',
-        'ne' => 'nep',
-        'ng' => 'ndo',
-        'nl' => 'nld',
-        'nn' => 'nno',
-        'nr' => 'nbl',
         'nv' => 'nav',
+        'nr' => 'nbl',
+        'nd' => 'nde',
+        'ng' => 'ndo',
+        'ne' => 'nep',
+        'nn' => 'nno',
+        'nb' => 'nob',
         'ny' => 'nya',
         'oc' => 'oci',
         'oj' => 'oji',
-        'om' => 'orm',
         'or' => 'ori',
         'os' => 'oss',
         'pa' => 'pan',
+        'ps' => 'pus',
         'pi' => 'pli',
         'pl' => 'pol',
-        'ps' => 'pus',
         'pt' => 'por',
         'qu' => 'que',
         'rm' => 'roh',
-        'rn' => 'run',
         'ro' => 'ron',
+        'rn' => 'run',
         'ru' => 'rus',
-        'rw' => 'kin',
-        'sa' => 'san',
-        'sc' => 'srd',
-        'sd' => 'snd',
-        'se' => 'sme',
         'sg' => 'sag',
+        'sa' => 'san',
+        'sr' => 'srp',
         'si' => 'sin',
         'sk' => 'slk',
         'sl' => 'slv',
+        'se' => 'sme',
         'sm' => 'smo',
         'sn' => 'sna',
+        'sd' => 'snd',
         'so' => 'som',
-        'sq' => 'sqi',
-        'sr' => 'srp',
-        'ss' => 'ssw',
         'st' => 'sot',
+        'es' => 'spa',
+        'sc' => 'srd',
+        'ss' => 'ssw',
         'su' => 'sun',
-        'sv' => 'swe',
         'sw' => 'swa',
+        'sv' => 'swe',
+        'ty' => 'tah',
         'ta' => 'tam',
+        'tt' => 'tat',
         'te' => 'tel',
         'tg' => 'tgk',
         'th' => 'tha',
         'ti' => 'tir',
-        'tk' => 'tuk',
-        'tn' => 'tsn',
         'to' => 'ton',
-        'tr' => 'tur',
+        'tn' => 'tsn',
         'ts' => 'tso',
-        'tt' => 'tat',
-        'ty' => 'tah',
+        'tk' => 'tuk',
+        'tr' => 'tur',
         'ug' => 'uig',
         'uk' => 'ukr',
         'ur' => 'urd',
@@ -813,7 +824,6 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
         'yi' => 'yid',
         'yo' => 'yor',
         'za' => 'zha',
-        'zh' => 'zho',
         'zu' => 'zul',
     );
 
@@ -848,7 +858,8 @@ abstract class AbstractLanguageDataProviderTest extends AbstractDataProviderTest
 
         sort($languages);
 
-        $this->assertEquals(static::$languages, $languages);
+        $this->assertNotEmpty($languages);
+        $this->assertEmpty(array_diff($languages, static::$languages));
     }
 
     public function testGetNamesDefaultLocale()
