@@ -11,15 +11,18 @@
 
 namespace Symfony\Component\Intl\Tests\DateFormatter;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\Intl\DateFormatter\IntlDateFormatter;
 use Symfony\Component\Intl\Globals\IntlGlobals;
+use Symfony\Component\Intl\Intl;
+use Symfony\Component\Intl\Util\IcuVersion;
 
 /**
  * Test case for IntlDateFormatter implementations.
  *
  * @author Bernhard Schussek <bschussek@gmail.com>
  */
-abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
+abstract class AbstractIntlDateFormatterTest extends TestCase
 {
     protected function setUp()
     {
@@ -35,12 +38,7 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
     {
         $formatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT);
 
-        // In PHP 5.5 default timezone depends on `date_default_timezone_get()` method
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $this->assertEquals(date_default_timezone_get(), $formatter->getTimeZoneId());
-        } else {
-            $this->assertNull($formatter->getTimeZoneId());
-        }
+        $this->assertEquals(date_default_timezone_get(), $formatter->getTimeZoneId());
 
         $this->assertEquals(
             $this->getDateTime(0, $formatter->getTimeZoneId())->format('M j, Y, g:i A'),
@@ -63,6 +61,8 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
 
     public function formatProvider()
     {
+        $dateTime = new \DateTime('@0');
+
         $formatData = array(
             /* general */
             array('y-M-d', 0, '1970-1-1'),
@@ -228,33 +228,38 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
             array('s', 3601, '1'),
             array('s', 3630, '30'),
             array('s', 43200, '0'), // 12 hours
-
-            // general
-            array("yyyy.MM.dd 'at' HH:mm:ss zzz", 0, '1970.01.01 at 00:00:00 GMT'),
-            array('K:mm a, z', 0, '0:00 AM, GMT'),
-
-            // timezone
-            array('z', 0, 'GMT'),
-            array('zz', 0, 'GMT'),
-            array('zzz', 0, 'GMT'),
-            array('zzzz', 0, 'GMT'),
-            array('zzzzz', 0, 'GMT'),
         );
 
-        $dateTime = new \DateTime('@0');
-
         /* general, DateTime */
         $formatData[] = array('y-M-d', $dateTime, '1970-1-1');
         $formatData[] = array("EEE, MMM d, ''yy", $dateTime, "Thu, Jan 1, '70");
         $formatData[] = array('h:mm a', $dateTime, '12:00 AM');
         $formatData[] = array('yyyyy.MMMM.dd hh:mm aaa', $dateTime, '01970.January.01 12:00 AM');
 
-        $formatData[] = array("yyyy.MM.dd 'at' HH:mm:ss zzz", $dateTime, '1970.01.01 at 00:00:00 GMT');
-        $formatData[] = array('K:mm a, z', $dateTime, '0:00 AM, GMT');
+        if (IcuVersion::compare(Intl::getIcuVersion(), '59.1', '>=', 1)) {
+            // Before ICU 59.1 GMT was used instead of UTC
+            $formatData[] = array("yyyy.MM.dd 'at' HH:mm:ss zzz", 0, '1970.01.01 at 00:00:00 UTC');
+            $formatData[] = array('K:mm a, z', 0, '0:00 AM, UTC');
+            $formatData[] = array("yyyy.MM.dd 'at' HH:mm:ss zzz", $dateTime, '1970.01.01 at 00:00:00 UTC');
+            $formatData[] = array('K:mm a, z', $dateTime, '0:00 AM, UTC');
+        }
 
         return $formatData;
     }
 
+    /**
+     * @requires PHP 5.5.10
+     */
+    public function testFormatUtcAndGmtAreSplit()
+    {
+        $pattern = "yyyy.MM.dd 'at' HH:mm:ss zzz";
+        $gmtFormatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT, 'GMT', IntlDateFormatter::GREGORIAN, $pattern);
+        $utcFormatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT, 'UTC', IntlDateFormatter::GREGORIAN, $pattern);
+
+        $this->assertSame('1970.01.01 at 00:00:00 GMT', $gmtFormatter->format(new \DateTime('@0')));
+        $this->assertSame('1970.01.01 at 00:00:00 UTC', $utcFormatter->format(new \DateTime('@0')));
+    }
+
     /**
      * @dataProvider formatErrorProvider
      */
@@ -269,18 +274,8 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
 
     public function formatErrorProvider()
     {
-        // With PHP 5.5 IntlDateFormatter accepts empty values ('0')
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            return array(
-                array('y-M-d', 'foobar', 'datefmt_format: string \'foobar\' is not numeric, which would be required for it to be a valid date: U_ILLEGAL_ARGUMENT_ERROR'),
-            );
-        }
-
-        $message = 'datefmt_format: takes either an array or an integer timestamp value or a DateTime object: U_ILLEGAL_ARGUMENT_ERROR';
-
         return array(
-            array('y-M-d', '0', $message),
-            array('y-M-d', 'foobar', $message),
+            array('y-M-d', 'foobar', 'datefmt_format: string \'foobar\' is not numeric, which would be required for it to be a valid date: U_ILLEGAL_ARGUMENT_ERROR'),
         );
     }
 
@@ -322,26 +317,83 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
             array(0, 'Pacific/Fiji', '1970-01-01 12:00:00'),
         );
 
-        // As of PHP 5.5, intl ext no longer fallbacks invalid time zones to UTC
-        if (PHP_VERSION_ID < 50500 && !(extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            // When time zone not exists, uses UTC by default
-            $data[] = array(0, 'Foo/Bar', '1970-01-01 00:00:00');
-            $data[] = array(0, 'UTC+04:30', '1970-01-01 00:00:00');
-            $data[] = array(0, 'UTC+04:AA', '1970-01-01 00:00:00');
+        return $data;
+    }
+
+    /**
+     * @dataProvider formatTimezoneProvider
+     * @requires PHP 5.5.10
+     */
+    public function testFormatTimezone($pattern, $timezone, $expected)
+    {
+        $formatter = $this->getDefaultDateFormatter($pattern);
+        $formatter->setTimeZone(new \DateTimeZone($timezone));
+
+        $this->assertEquals($expected, $formatter->format(0));
+    }
+
+    public function formatTimezoneProvider()
+    {
+        $cases = array(
+            array('z', 'GMT', 'GMT'),
+            array('zz', 'GMT', 'GMT'),
+            array('zzz', 'GMT', 'GMT'),
+            array('zzzz', 'GMT', 'Greenwich Mean Time'),
+            array('zzzzz', 'GMT', 'Greenwich Mean Time'),
+
+            array('z', 'Etc/GMT', 'GMT'),
+            array('zz', 'Etc/GMT', 'GMT'),
+            array('zzz', 'Etc/GMT', 'GMT'),
+            array('zzzz', 'Etc/GMT', 'Greenwich Mean Time'),
+            array('zzzzz', 'Etc/GMT', 'Greenwich Mean Time'),
+
+            array('z', 'Etc/GMT+3', 'GMT-3'),
+            array('zz', 'Etc/GMT+3', 'GMT-3'),
+            array('zzz', 'Etc/GMT+3', 'GMT-3'),
+            array('zzzz', 'Etc/GMT+3', 'GMT-03:00'),
+            array('zzzzz', 'Etc/GMT+3', 'GMT-03:00'),
+
+            array('z', 'UTC', 'UTC'),
+            array('zz', 'UTC', 'UTC'),
+            array('zzz', 'UTC', 'UTC'),
+            array('zzzz', 'UTC', 'Coordinated Universal Time'),
+            array('zzzzz', 'UTC', 'Coordinated Universal Time'),
+
+            array('z', 'Etc/UTC', 'UTC'),
+            array('zz', 'Etc/UTC', 'UTC'),
+            array('zzz', 'Etc/UTC', 'UTC'),
+            array('zzzz', 'Etc/UTC', 'Coordinated Universal Time'),
+            array('zzzzz', 'Etc/UTC', 'Coordinated Universal Time'),
+
+            array('z', 'Etc/Universal', 'UTC'),
+            array('z', 'Etc/Zulu', 'UTC'),
+            array('z', 'Etc/UCT', 'UTC'),
+            array('z', 'Etc/Greenwich', 'GMT'),
+            array('zzzzz', 'Etc/Universal', 'Coordinated Universal Time'),
+            array('zzzzz', 'Etc/Zulu', 'Coordinated Universal Time'),
+            array('zzzzz', 'Etc/UCT', 'Coordinated Universal Time'),
+            array('zzzzz', 'Etc/Greenwich', 'Greenwich Mean Time'),
+        );
+
+        if (!defined('HHVM_VERSION')) {
+            // these timezones are not considered valid in HHVM
+            $cases = array_merge($cases, array(
+                array('z', 'GMT+03:00', 'GMT+3'),
+                array('zz', 'GMT+03:00', 'GMT+3'),
+                array('zzz', 'GMT+03:00', 'GMT+3'),
+                array('zzzz', 'GMT+03:00', 'GMT+03:00'),
+                array('zzzzz', 'GMT+03:00', 'GMT+03:00'),
+            ));
         }
 
-        return $data;
+        return $cases;
     }
 
     public function testFormatWithGmtTimezone()
     {
         $formatter = $this->getDefaultDateFormatter('zzzz');
 
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $formatter->setTimeZone('GMT+03:00');
-        } else {
-            $formatter->setTimeZoneId('GMT+03:00');
-        }
+        $formatter->setTimeZone('GMT+03:00');
 
         $this->assertEquals('GMT+03:00', $formatter->format(0));
     }
@@ -350,11 +402,7 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
     {
         $formatter = $this->getDefaultDateFormatter('zzzz');
 
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $formatter->setTimeZone('GMT+00:30');
-        } else {
-            $formatter->setTimeZoneId('GMT+00:30');
-        }
+        $formatter->setTimeZone('GMT+00:30');
 
         $this->assertEquals('GMT+00:30', $formatter->format(0));
     }
@@ -363,11 +411,7 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
     {
         $formatter = $this->getDefaultDateFormatter('zzzz');
 
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $formatter->setTimeZone('Pacific/Fiji');
-        } else {
-            $formatter->setTimeZoneId('Pacific/Fiji');
-        }
+        $formatter->setTimeZone('Pacific/Fiji');
 
         $this->assertEquals('Fiji Standard Time', $formatter->format(0));
     }
@@ -383,20 +427,19 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
         );
     }
 
+    /**
+     * @requires PHP 5.5.10
+     */
     public function testFormatWithDateTimeZoneGmt()
     {
-        if (PHP_VERSION_ID < 50500 && !(extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $this->markTestSkipped('Only in PHP 5.5+ IntlDateFormatter allows to use DateTimeZone objects.');
-        }
-
-        $formatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT, new \DateTimeZone('GMT'), IntlDateFormatter::GREGORIAN, 'zzzz');
+        $formatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT, new \DateTimeZone('GMT'), IntlDateFormatter::GREGORIAN, 'zzz');
 
         $this->assertEquals('GMT', $formatter->format(0));
     }
 
     public function testFormatWithDateTimeZoneGmtOffset()
     {
-        if (defined('HHVM_VERSION_ID') || PHP_VERSION_ID <= 50509) {
+        if (defined('HHVM_VERSION_ID') || \PHP_VERSION_ID <= 50509) {
             $this->markTestSkipped('DateTimeZone GMT offsets are supported since 5.5.10. See https://github.com/facebook/hhvm/issues/5875 for HHVM.');
         }
 
@@ -410,44 +453,14 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
         if (!extension_loaded('intl')) {
             $this->markTestSkipped('Extension intl is required.');
         }
-        if (PHP_VERSION_ID < 50500 && !method_exists('IntlDateFormatter', 'setTimeZone')) {
-            $this->markTestSkipped('Only in PHP 5.5+ IntlDateFormatter allows to use DateTimeZone objects.');
-        }
 
         $formatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT, \IntlTimeZone::createTimeZone('GMT+03:00'), IntlDateFormatter::GREGORIAN, 'zzzz');
 
         $this->assertEquals('GMT+03:00', $formatter->format(0));
     }
 
-    public function testFormatWithTimezoneFromEnvironmentVariable()
-    {
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $this->markTestSkipped('IntlDateFormatter in PHP 5.5 no longer depends on TZ environment.');
-        }
-
-        $tz = getenv('TZ');
-        putenv('TZ=Europe/London');
-
-        $formatter = $this->getDateFormatter('en', IntlDateFormatter::MEDIUM, IntlDateFormatter::SHORT);
-        $formatter->setPattern('yyyy-MM-dd HH:mm:ss');
-
-        $this->assertEquals(
-            $this->getDateTime(0, 'Europe/London')->format('Y-m-d H:i:s'),
-            $formatter->format(0)
-        );
-
-        $this->assertEquals('Europe/London', getenv('TZ'));
-
-        // Restores TZ.
-        putenv('TZ='.$tz);
-    }
-
     public function testFormatWithTimezoneFromPhp()
     {
-        if (PHP_VERSION_ID < 50500 && !(extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $this->markTestSkipped('Only in PHP 5.5 IntlDateFormatter depends on default timezone (`date_default_timezone_get()`).');
-        }
-
         $tz = date_default_timezone_get();
         date_default_timezone_set('Europe/London');
 
@@ -481,8 +494,8 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
             array(0, IntlDateFormatter::LONG, IntlDateFormatter::NONE, 'January 1, 1970'),
             array(0, IntlDateFormatter::MEDIUM, IntlDateFormatter::NONE, 'Jan 1, 1970'),
             array(0, IntlDateFormatter::SHORT, IntlDateFormatter::NONE, '1/1/70'),
-            array(0, IntlDateFormatter::NONE, IntlDateFormatter::FULL, '12:00:00 AM GMT'),
-            array(0, IntlDateFormatter::NONE, IntlDateFormatter::LONG, '12:00:00 AM GMT'),
+            array(0, IntlDateFormatter::NONE, IntlDateFormatter::FULL, '12:00:00 AM Coordinated Universal Time'),
+            array(0, IntlDateFormatter::NONE, IntlDateFormatter::LONG, '12:00:00 AM UTC'),
             array(0, IntlDateFormatter::NONE, IntlDateFormatter::MEDIUM, '12:00:00 AM'),
             array(0, IntlDateFormatter::NONE, IntlDateFormatter::SHORT, '12:00 AM'),
         );
@@ -857,7 +870,9 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
         $position = null;
         $formatter = $this->getDefaultDateFormatter('y');
         $this->assertSame(0, $formatter->parse('1970', $position));
-        $this->assertNull($position);
+        // Since $position is not supported by the Symfony implementation, the following won't work.
+        // The intl implementation works this way since 60.2.
+        // $this->assertSame(4, $position);
     }
 
     public function testSetPattern()
@@ -874,28 +889,22 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
     {
         $formatter = $this->getDefaultDateFormatter();
 
-        if (PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
-            $formatter->setTimeZone($timeZoneId);
-        } else {
-            $formatter->setTimeZoneId($timeZoneId);
-        }
+        $formatter->setTimeZone($timeZoneId);
 
         $this->assertEquals($expectedTimeZoneId, $formatter->getTimeZoneId());
     }
 
     public function setTimeZoneIdProvider()
     {
-        $isPhp55 = PHP_VERSION_ID >= 50500 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'));
-
         return array(
             array('UTC', 'UTC'),
             array('GMT', 'GMT'),
             array('GMT-03:00', 'GMT-03:00'),
             array('Europe/Zurich', 'Europe/Zurich'),
-            array(null, $isPhp55 ? date_default_timezone_get() : null),
-            array('Foo/Bar', $isPhp55 ? 'UTC' : 'Foo/Bar'),
-            array('GMT+00:AA', $isPhp55 ? 'UTC' : 'GMT+00:AA'),
-            array('GMT+00AA', $isPhp55 ? 'UTC' : 'GMT+00AA'),
+            array(null, date_default_timezone_get()),
+            array('Foo/Bar', 'UTC'),
+            array('GMT+00:AA', 'UTC'),
+            array('GMT+00AA', 'UTC'),
         );
     }
 
@@ -908,9 +917,7 @@ abstract class AbstractIntlDateFormatterTest extends \PHPUnit_Framework_TestCase
     {
         $dateTime = new \DateTime();
         $dateTime->setTimestamp(null === $timestamp ? time() : $timestamp);
-        if (null !== $timeZone) {
-            $dateTime->setTimezone(new \DateTimeZone($timeZone));
-        }
+        $dateTime->setTimezone(new \DateTimeZone($timeZone ?: getenv('TZ') ?: 'UTC'));
 
         return $dateTime;
     }
