@@ -19,14 +19,14 @@ use Symfony\Component\Serializer\Annotation\Groups;
 class Dummy extends ParentDummy
 {
     /**
-     * @var string This is bar.
+     * @var string This is bar
      */
     private $bar;
 
     /**
      * Should be used.
      *
-     * @var int Should be ignored.
+     * @var int Should be ignored
      */
     protected $baz;
 
@@ -51,6 +51,45 @@ class Dummy extends ParentDummy
      */
     public $B;
 
+    /**
+     * @var int
+     */
+    protected $Id;
+
+    /**
+     * @var string
+     */
+    public $Guid;
+
+    /**
+     * Nullable array.
+     *
+     * @var array|null
+     */
+    public $g;
+
+    /**
+     * This should not be removed.
+     *
+     * @var
+     */
+    public $emptyVar;
+
+    public static function getStatic()
+    {
+    }
+
+    /**
+     * @return string
+     */
+    public static function staticGetter()
+    {
+    }
+
+    public static function staticSetter(\DateTime $d)
+    {
+    }
+
     /**
      * A.
      *
@@ -77,4 +116,15 @@ class Dummy extends ParentDummy
     public function getDOB()
     {
     }
+
+    /**
+     * @return int
+     */
+    public function getId()
+    {
+    }
+
+    public function get123()
+    {
+    }
 }
