@@ -24,12 +24,17 @@ use Symfony\Component\Security\Core\Authentication\AuthenticationManagerInterfac
  */
 interface AuthenticationProviderInterface extends AuthenticationManagerInterface
 {
+    /**
+     * Use this constant for not provided username.
+     *
+     * @var string
+     */
+    const USERNAME_NONE_PROVIDED = 'NONE_PROVIDED';
+
     /**
      * Checks whether this provider supports the given token.
      *
-     * @param TokenInterface $token A TokenInterface instance
-     *
-     * @return bool    true if the implementation supports the Token, false otherwise
+     * @return bool true if the implementation supports the Token, false otherwise
      */
-     public function supports(TokenInterface $token);
+    public function supports(TokenInterface $token);
 }
