@@ -25,15 +25,13 @@ class AccessMap implements AccessMapInterface
     private $map = array();
 
     /**
-     * Constructor.
-     *
      * @param RequestMatcherInterface $requestMatcher A RequestMatcherInterface instance
-     * @param array                   $roles          An array of roles needed to access the resource
+     * @param array                   $attributes     An array of attributes to pass to the access decision manager (like roles)
      * @param string|null             $channel        The channel to enforce (http, https, or null)
      */
-    public function add(RequestMatcherInterface $requestMatcher, array $roles = array(), $channel = null)
+    public function add(RequestMatcherInterface $requestMatcher, array $attributes = array(), $channel = null)
     {
-        $this->map[] = array($requestMatcher, $roles, $channel);
+        $this->map[] = array($requestMatcher, $attributes, $channel);
     }
 
     /**
