@@ -28,6 +28,8 @@ use Symfony\Component\Security\Core\Exception\AuthenticationException;
  * DigestAuthenticationListener implements Digest HTTP authentication.
  *
  * @author Fabien Potencier <fabien@symfony.com>
+ *
+ * @deprecated since 3.4, to be removed in 4.0
  */
 class DigestAuthenticationListener implements ListenerInterface
 {
@@ -39,6 +41,8 @@ class DigestAuthenticationListener implements ListenerInterface
 
     public function __construct(TokenStorageInterface $tokenStorage, UserProviderInterface $provider, $providerKey, DigestAuthenticationEntryPoint $authenticationEntryPoint, LoggerInterface $logger = null)
     {
+        @trigger_error(sprintf('The %s class and the whole HTTP digest authentication system is deprecated since Symfony 3.4 and will be removed in 4.0.', __CLASS__), E_USER_DEPRECATED);
+
         if (empty($providerKey)) {
             throw new \InvalidArgumentException('$providerKey must not be empty.');
         }
@@ -53,8 +57,6 @@ class DigestAuthenticationListener implements ListenerInterface
     /**
      * Handles digest authentication.
      *
-     * @param GetResponseEvent $event A GetResponseEvent instance
-     *
      * @throws AuthenticationServiceException
      */
     public function handle(GetResponseEvent $event)
@@ -78,7 +80,7 @@ class DigestAuthenticationListener implements ListenerInterface
         }
 
         try {
-            $digestAuth->validateAndDecode($this->authenticationEntryPoint->getKey(), $this->authenticationEntryPoint->getRealmName());
+            $digestAuth->validateAndDecode($this->authenticationEntryPoint->getSecret(), $this->authenticationEntryPoint->getRealmName());
         } catch (BadCredentialsException $e) {
             $this->fail($event, $request, $e);
 
@@ -137,6 +139,9 @@ class DigestAuthenticationListener implements ListenerInterface
     }
 }
 
+/**
+ * @deprecated since 3.4, to be removed in 4.0.
+ */
 class DigestData
 {
     private $elements = array();
@@ -145,6 +150,8 @@ class DigestData
 
     public function __construct($header)
     {
+        @trigger_error(sprintf('The %s class and the whole HTTP digest authentication system is deprecated since Symfony 3.4 and will be removed in 4.0.', __CLASS__), E_USER_DEPRECATED);
+
         $this->header = $header;
         preg_match_all('/(\w+)=("((?:[^"\\\\]|\\\\.)+)"|([^\s,$]+))/', $header, $matches, PREG_SET_ORDER);
         foreach ($matches as $match) {
@@ -205,7 +212,7 @@ class DigestData
         } elseif ('auth' === $this->elements['qop']) {
             $digest .= ':'.$this->elements['nc'].':'.$this->elements['cnonce'].':'.$this->elements['qop'];
         } else {
-            throw new \InvalidArgumentException('This method does not support a qop: "%s".', $this->elements['qop']);
+            throw new \InvalidArgumentException(sprintf('This method does not support a qop: "%s".', $this->elements['qop']));
         }
         $digest .= ':'.$a2Md5;
 
