@@ -30,8 +30,6 @@ class CacheLoader extends Loader
     protected $dir;
 
     /**
-     * Constructor.
-     *
      * @param LoaderInterface $loader A Loader instance
      * @param string          $dir    The directory where to store the cache files
      */
@@ -44,8 +42,6 @@ class CacheLoader extends Loader
     /**
      * Loads a template.
      *
-     * @param TemplateReferenceInterface $template A template
-     *
      * @return Storage|bool false if the template cannot be loaded, a Storage instance otherwise
      */
     public function load(TemplateReferenceInterface $template)
@@ -58,9 +54,6 @@ class CacheLoader extends Loader
         if (is_file($path)) {
             if (null !== $this->logger) {
                 $this->logger->debug('Fetching template from cache.', array('name' => $template->get('name')));
-            } elseif (null !== $this->debugger) {
-                // just for BC, to be removed in 3.0
-                $this->debugger->log(sprintf('Fetching template "%s" from cache.', $template->get('name')));
             }
 
             return new FileStorage($path);
@@ -80,9 +73,6 @@ class CacheLoader extends Loader
 
         if (null !== $this->logger) {
             $this->logger->debug('Storing template in cache.', array('name' => $template->get('name')));
-        } elseif (null !== $this->debugger) {
-            // just for BC, to be removed in 3.0
-            $this->debugger->log(sprintf('Storing template "%s" in cache.', $template->get('name')));
         }
 
         return new FileStorage($path);
