@@ -11,16 +11,18 @@
 
 namespace Symfony\Component\Templating\Tests\Loader;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\Templating\Loader\Loader;
 use Symfony\Component\Templating\TemplateReferenceInterface;
 
-class LoaderTest extends \PHPUnit_Framework_TestCase
+class LoaderTest extends TestCase
 {
-    public function testGetSetDebugger()
+    public function testGetSetLogger()
     {
         $loader = new ProjectTemplateLoader4();
-        $loader->setDebugger($debugger = new \Symfony\Component\Templating\Tests\Fixtures\ProjectTemplateDebugger());
-        $this->assertTrue($loader->getDebugger() === $debugger, '->setDebugger() sets the debugger instance');
+        $logger = $this->getMockBuilder('Psr\Log\LoggerInterface')->getMock();
+        $loader->setLogger($logger);
+        $this->assertSame($logger, $loader->getLogger(), '->setLogger() sets the logger instance');
     }
 }
 
@@ -30,6 +32,11 @@ class ProjectTemplateLoader4 extends Loader
     {
     }
 
+    public function getLogger()
+    {
+        return $this->logger;
+    }
+
     public function getDebugger()
     {
         return $this->debugger;
