@@ -11,9 +11,10 @@
 
 namespace Symfony\Component\Validator\Tests;
 
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\Validator\ConstraintViolation;
 
-class ConstraintViolationTest extends \PHPUnit_Framework_TestCase
+class ConstraintViolationTest extends TestCase
 {
     public function testToStringHandlesArrays()
     {
@@ -26,7 +27,7 @@ class ConstraintViolationTest extends \PHPUnit_Framework_TestCase
             null
         );
 
-        $expected = <<<EOF
+        $expected = <<<'EOF'
 Root.property.path:
     Array
 EOF;
@@ -40,12 +41,12 @@ EOF;
             '42 cannot be used here',
             'this is the message template',
             array(),
-            array('some_value' =>  42),
+            array('some_value' => 42),
             'some_value',
             null
         );
 
-        $expected = <<<EOF
+        $expected = <<<'EOF'
 Array.some_value:
     42 cannot be used here
 EOF;
