@@ -13,15 +13,10 @@ namespace Symfony\Component\Validator\Tests\Constraints;
 
 use Symfony\Component\Validator\Constraints\Iban;
 use Symfony\Component\Validator\Constraints\IbanValidator;
-use Symfony\Component\Validator\Validation;
+use Symfony\Component\Validator\Test\ConstraintValidatorTestCase;
 
-class IbanValidatorTest extends AbstractConstraintValidatorTest
+class IbanValidatorTest extends ConstraintValidatorTestCase
 {
-    protected function getApiVersion()
-    {
-        return Validation::API_VERSION_2_5;
-    }
-
     protected function createValidator()
     {
         return new IbanValidator();
@@ -68,6 +63,9 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('BE62 5100 0754 7061'), //Belgium
             array('BA39 1290 0794 0102 8494'), //Bosnia and Herzegovina
             array('BG80 BNBG 9661 1020 3456 78'), //Bulgaria
+            array('BY 13 NBRB 3600 900000002Z00AB00'), //Belarus
+            array('BY13 NBRB 3600 900000002Z00AB00'), //Belarus
+            array('BY22NB23324232T78YR7823HR32U'), //Belarus
             array('HR12 1001 0051 8630 0016 0'), //Croatia
             array('CY17 0020 0128 0000 0012 0052 7600'), //Cyprus
             array('CZ65 0800 0000 1920 0014 5399'), //Czech Republic
@@ -118,7 +116,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
 
             //Extended country list
             //http://www.nordea.com/Our+services/International+products+and+services/Cash+Management/IBAN+countries/908462.html
-            // http://www.swift.com/dsp/resources/documents/IBAN_Registry.pdf
+            // https://www.swift.com/sites/default/files/resources/iban_registry.pdf
             array('AO06000600000100037131174'), //Angola
             array('AZ21NABZ00000000137010001944'), //Azerbaijan
             array('BH29BMAG1299123456BH00'), //Bahrain
@@ -132,7 +130,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('CV64000300004547069110176'), //Cape Verde
             array('FR7630007000110009970004942'), //Central African Republic
             array('CG5230011000202151234567890'), //Congo
-            array('CR0515202001026284066'), //Costa Rica
+            array('CR05015202001026284066'), //Costa Rica
             array('DO28BAGR00000001212453611324'), //Dominican Republic
             array('GT82TRAJ01020000001210029690'), //Guatemala
             array('IR580540105180021273113007'), //Iran
@@ -156,6 +154,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('TL380080012345678910157'), //Timor-Leste
             array('TN5914207207100707129648'), //Tunisia
             array('TR330006100519786457841326'), //Turkey
+            array('UA213223130000026007233566001'), //Ukraine
             array('AE260211000000230064016'), //United Arab Emirates
         );
     }
@@ -182,6 +181,10 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('BA39 1290 0794 0102 8494 4'), //Bosnia and Herzegovina
             array('BG80 BNBG 9661 1020 3456 7'), //Bulgaria
             array('BG80 B2BG 9661 1020 3456 78'), //Bulgaria
+            array('BY 13 NBRB 3600 900000002Z00AB001'), //Belarus
+            array('BY 13 NBRB 3600 900000002Z00AB0'), //Belarus
+            array('BYRO NBRB 3600 900000002Z00AB0'), //Belarus
+            array('BY 13 3600 NBRB 900000002Z00AB05'), //Belarus
             array('HR12 1001 0051 8630 0016 01'), //Croatia
             array('CY17 0020 0128 0000 0012 0052 7600 1'), //Cyprus
             array('CZ65 0800 0000 1920 0014 5399 1'), //Czech Republic
@@ -245,6 +248,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('FR76300070001100099700049421'), //Central African Republic
             array('CG52300110002021512345678901'), //Congo
             array('CR05152020010262840661'), //Costa Rica
+            array('CR0515202001026284066'), //Costa Rica
             array('DO28BAGR000000012124536113241'), //Dominican Republic
             array('GT82TRAJ010200000012100296901'), //Guatemala
             array('IR5805401051800212731130071'), //Iran
@@ -268,6 +272,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('TL3800800123456789101571'), //Timor-Leste
             array('TN59142072071007071296481'), //Tunisia
             array('TR3300061005197864578413261'), //Turkey
+            array('UA21AAAA1300000260072335660012'), //Ukraine
             array('AE2602110000002300640161'), //United Arab Emirates
         );
     }
@@ -291,6 +296,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('BE62 5100 0754 7062'), //Belgium
             array('BA39 1290 0794 0102 8495'), //Bosnia and Herzegovina
             array('BG80 BNBG 9661 1020 3456 79'), //Bulgaria
+            array('BY90 NBRB 3600 900000002Z00AB00'), //Belarus
             array('HR12 1001 0051 8630 0016 1'), //Croatia
             array('CY17 0020 0128 0000 0012 0052 7601'), //Cyprus
             array('CZ65 0800 0000 1920 0014 5398'), //Czech Republic
@@ -353,7 +359,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('CV64000300004547069110177'), //Cape Verde
             array('FR7630007000110009970004943'), //Central African Republic
             array('CG5230011000202151234567891'), //Congo
-            array('CR0515202001026284067'), //Costa Rica
+            array('CR96042332432534543564'), //Costa Rica
             array('DO28BAGR00000001212453611325'), //Dominican Republic
             array('GT82TRAJ01020000001210029691'), //Guatemala
             array('IR580540105180021273113008'), //Iran
@@ -377,6 +383,7 @@ class IbanValidatorTest extends AbstractConstraintValidatorTest
             array('TL380080012345678910158'), //Timor-Leste
             array('TN5914207207100707129649'), //Tunisia
             array('TR330006100519786457841327'), //Turkey
+            array('UA213223130000026007233566002'), //Ukraine
             array('AE260211000000230064017'), //United Arab Emirates
         );
     }
