@@ -12,6 +12,7 @@
 namespace Symfony\Component\Validator\Tests\Mapping\Loader;
 
 use Doctrine\Common\Annotations\AnnotationReader;
+use PHPUnit\Framework\TestCase;
 use Symfony\Component\Validator\Constraints\All;
 use Symfony\Component\Validator\Constraints\Callback;
 use Symfony\Component\Validator\Constraints\Choice;
@@ -19,11 +20,12 @@ use Symfony\Component\Validator\Constraints\Collection;
 use Symfony\Component\Validator\Constraints\NotNull;
 use Symfony\Component\Validator\Constraints\Range;
 use Symfony\Component\Validator\Constraints\IsTrue;
+use Symfony\Component\Validator\Constraints\Valid;
 use Symfony\Component\Validator\Mapping\ClassMetadata;
 use Symfony\Component\Validator\Mapping\Loader\AnnotationLoader;
 use Symfony\Component\Validator\Tests\Fixtures\ConstraintA;
 
-class AnnotationLoaderTest extends \PHPUnit_Framework_TestCase
+class AnnotationLoaderTest extends TestCase
 {
     public function testLoadClassMetadataReturnsTrueIfSuccessful()
     {
@@ -67,8 +69,10 @@ class AnnotationLoaderTest extends \PHPUnit_Framework_TestCase
             'message' => 'Must be one of %choices%',
             'choices' => array('A', 'B'),
         )));
+        $expected->addPropertyConstraint('childA', new Valid());
+        $expected->addPropertyConstraint('childB', new Valid());
         $expected->addGetterConstraint('lastName', new NotNull());
-        $expected->addGetterConstraint('valid', new IsTrue());
+        $expected->addGetterMethodConstraint('valid', 'isValid', new IsTrue());
         $expected->addGetterConstraint('permissions', new IsTrue());
 
         // load reflection class so that the comparison passes
@@ -137,8 +141,10 @@ class AnnotationLoaderTest extends \PHPUnit_Framework_TestCase
             'message' => 'Must be one of %choices%',
             'choices' => array('A', 'B'),
         )));
+        $expected->addPropertyConstraint('childA', new Valid());
+        $expected->addPropertyConstraint('childB', new Valid());
         $expected->addGetterConstraint('lastName', new NotNull());
-        $expected->addGetterConstraint('valid', new IsTrue());
+        $expected->addGetterMethodConstraint('valid', 'isValid', new IsTrue());
         $expected->addGetterConstraint('permissions', new IsTrue());
 
         // load reflection class so that the comparison passes
